import java.awt.*;

public class DrawPainter extends PaintPainter {

	Stroke stroke;

	public DrawPainter(Paint paint, Stroke stroke) throws NullPointerException {
		// EFFECTS: If paint or stroke is null throws NullPointerException;
		//   else initializes this with the specified paint and stroke.
		super(paint);
		if (stroke==null) throw new NullPointerException();
		this.stroke = stroke;
	} // DrawPainter

	public DrawPainter(Paint paint) throws NullPointerException {
		// EFFECTS: If paint is null throws NullPointerException;
		//   else initializes this with the specified paint and default stroke
		//   (one-pixel wide),
		this(paint,new BasicStroke());
	} // DrawPainter

	public DrawPainter() {
		// EFFECTS: Initializes this with the default paint (white)
		//    and default stroke (one-pixel wide),
		this(DefaultPaint,new BasicStroke());
	} // DrawPainter

	// stroke property
	public void setStroke(Stroke newStroke) throws NullPointerException {
		// MODIFIES: this.
		// EFFECTS: If stroke is null throws NullPointerException;
		//   else sets stroke to newStroke..
		if (stroke==null) throw new NullPointerException();
		this.stroke = newStroke;
	} // setStroke

	public Stroke getStroke() {
		// EFFECTS: Returns the current stroke.
		return stroke;
	} // getStroke

	public void paint(Graphics2D g2, Shape shape) {
		// REQUIRES: g2 and geometry are not null.
		// EFFECTS: Paints geometry into redering context g2by drawing
		//   the outline using th current paint and stroke.
		Shape shapeStroke = stroke.createStrokedShape(shape);
		g2.setPaint(getPaint());
		g2.fill(shapeStroke);
	} // paint

} // DrawPainter