import java.io.*;

/** File filter that allows directories and files with a given extension
* to be shown by file choosers.*/
public class ECFFileFilter extends javax.swing.filechooser.FileFilter {

	/** The extension of files that shall be shown. */
	protected String szExtension;

	/** Creates a file filter.
	* @param szExtension The extension of files that shall be shown. */
	public ECFFileFilter(String szExtension) {
		this.szExtension = szExtension;
	} // ECFFileFilter

	/** Tests if a file shall be shown by file choosers,<br> directories and
	* files with extension <code>szExtension</code> are accepted.
	* @param f The file to be tested.
	* @return true if file shall be shown, else false. */
	public boolean accept(File f) {
//		System.out.println("ECFFileFilter.accept: f=" + f);

		if (f==null) return false;
	    if (f.isDirectory()) {
		return true;
	    }

	    String extension = ECFFileFilterUtils.getExtension(f);
	    if (extension != null) {
			if (extension.equals(szExtension)) {
				return true;
			} else {
			    return false;
			}
	    } // if extension != null

	    return false;
	} // accept

	/** The description of files that are shown, appears in
	* the "File format:" combo box at the last line of the
	* file chooser. */
	public String getDescription() {
		return "*." + szExtension;
	} // getDescription

} // ECFFileFilter