import java.awt.*;        // Container, BorderLayout
import java.awt.event.*;  // WindowAdapter
import javax.swing.*;     // JFrame, JPanel
import javax.swing.plaf.basic.BasicProgressBarUI;
import java.io.*;         // FileWriter
import java.util.*;       // Vector
import java.text.*;       // DateFormat

public class ECFMainFrame extends JFrame implements ActionListener {

	protected static ECFMainFrame mainframe;
	protected String szPathTitle = " ";

	/** Get the name of the XML model file.
	* @return The name of the XML model file. */
	public static String getFileName() {
		try { System.out.println("ECFMainFrame.getFileName:");
			return chooserXML.getSelectedFile().getName();
		} catch(Exception e) { return "ECF.xml"; }
	} // getFileName

	/** Returns an ImageIcon, or null if the path was invalid. */
	protected static ImageIcon createImageIcon(String path,
	                                           String description) {
	    java.net.URL imgURL = ECFMainFrame.class.getResource(path);
	    if (imgURL != null) {
	        return new ImageIcon(imgURL, description);
	    } else {
	        System.err.println("Couldn't find file: " + path);
	        return null;
	    }
	} // createImageIcon

	/** Zoom combo box */
	private static JComboBox comboxZoom;
	/** Scales for model diagram zoom.*/
	private static double vrscale[] = {1.0,0.71,0.5};
	public static int getZoom() { return comboxZoom.getSelectedIndex(); }
	public static void setZoom(int iscale) {
		if (comboxZoom==null) return;
		comboxZoom.setSelectedIndex(iscale);
		ECFModel.scaleDisplay = vrscale[iscale];
	}

	protected GraphsPanel panelGraphs;
	protected ButtonsPanel panelButtons;
	protected AlertPanel panelAlert;
	protected TimeScrollPanel panelTimeScroll;
	private Container panelContent;
	// Choosers
	private static JFileChooser chooserXML;
	private static JFileChooser chooserDat;
	// Menu items
	private JMenuItem menuItemSaveXML;
	private JMenuItem menuItemOpenXML;
	private JMenuItem menuItemPrint;
	private JMenuItem menuItemPrintToPrinter;
	private JMenuItem menuItemPrintToPrinterNoHeading;
	private JMenuItem menuItemPrintToFile;
	private JMenuItem menuItemExportText;
	private JMenuItem menuItemExportExcel;
	private JMenuItem menuItemExportMatlab;
	private JMenuItem menuItemExportMaple;
	private JMenuItem menuItemExit;

	ECFMainFrame() {

		// Put an ECF icon in the frame
		ImageIcon icon = createImageIcon("ECFIcon.gif","ECF icon");
		setIconImage(icon.getImage());

		// Set frame properties
		mainframe = this;
		setTitle("ECF Version " + ECF.szVersionProgram + "  " + ECF.szDate);
		setSize(700,700);
		ECFUtil.center(this);

		// Menu bar
        setJMenuBar(createMenuBar());

		// Set layout fo control at the top and graphs panel at the remainder area.
		panelContent = getContentPane();
		panelContent.setLayout(new BorderLayout());

		// Create graphs panel first because reference is made in buttons panel.
		panelGraphs = new GraphsPanel(this);

		// Create top panel
		JPanel panelTop = new JPanel();
		panelTop.setLayout(new BoxLayout(panelTop, BoxLayout.Y_AXIS));

		// Button and alert panels side by side
		panelButtons = new ButtonsPanel(panelGraphs);
		panelAlert = new AlertPanel();
		JPanel panelButtonsAlert = new JPanel();
		BoxLayout boxlayoutButtonsAlert = new BoxLayout(panelButtonsAlert,BoxLayout.X_AXIS);
		panelButtonsAlert.setLayout(boxlayoutButtonsAlert);
		panelButtonsAlert.add(panelButtons);
		panelButtonsAlert.add(panelAlert);

		// Time and zoom panel
		JPanel panelTimeZoom = new JPanel();
		panelTimeZoom.setLayout(new BoxLayout(panelTimeZoom, BoxLayout.X_AXIS));
		// Time scroll panel
		panelTimeScroll = new TimeScrollPanel(panelAlert,panelGraphs);
		// Zoom combo box
		comboxZoom = new JComboBox();
		for (int iscale=0; iscale<vrscale.length; iscale++) {
			String szZoom = (new Integer((int)(100*vrscale[iscale]))).toString() + " %";
			comboxZoom.addItem(szZoom);
		} // while
		ECFModel.scaleDisplay = vrscale[0];
		comboxZoom.setSelectedIndex(0);
		comboxZoom.addActionListener(this);
		// Add time scroll and zoom side by side
		panelTimeZoom.add(panelTimeScroll);
		panelTimeZoom.add(new JLabel("  Zoom: "));
		panelTimeZoom.add(comboxZoom);

		// Set top panel content
		panelTop.add(panelButtonsAlert);
		panelTop.add(panelTimeZoom);

		// Set frame content
		panelContent.add(panelTop,BorderLayout.NORTH);
		panelContent.add(panelGraphs,BorderLayout.CENTER);

		show();
		panelGraphs.tileFrames(GraphsPanel.TILE_VERTICAL);

		// File chooserXML
		chooserXML = new JFileChooser();
		chooserXML.setFileFilter(new ECFFileFilter("xml"));
		chooserXML.setCurrentDirectory(new File("."));
		// File chooserDat
		chooserDat = new JFileChooser();
		chooserDat.addChoosableFileFilter(new ECFFileFilter("dat"));
		chooserDat.setCurrentDirectory(new File("."));

		// Key listener
		addKeyListener(ECF.modelOpen);

		// Close frame
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				askBeforeClosing(JOptionPane.YES_NO_OPTION);
				dispose();
				System.exit(0);
			} // windowClosing
		}); // addWindowListener

	} // ECFMainFrame

    private void askBeforeClosing(int optionType) {
		if (ECFModel.fModelWasChanged==true) {
				String szMsg = "Model was changed, save before exiting?";
				int messageType = JOptionPane.WARNING_MESSAGE;
				JOptionPane pane = new JOptionPane(szMsg,messageType,optionType);
			    JDialog dialog = pane.createDialog(this,"Save model before exit?");
			    dialog.show();
			    Object selectedValue = pane.getValue();
			    if (selectedValue != null) {
			    	// Dialog was not closed by cross in upper left corner
			   	 	int option = ((Integer)selectedValue).intValue();
			    	if(option == JOptionPane.YES_OPTION ) {
						saveXMLCode();
						dispose();
						System.exit(0);
					} else if(option == JOptionPane.CANCEL_OPTION) {
//						System.out.println("ECFMainFrame.askBeforeClosing: CANCEL_OPTION");
					} else {
						dispose();
						System.exit(0);
					} // option
				} // if/else selectedValue
			} else {
				dispose();
				System.exit(0);
			}// if fModelWasChanged
//		System.out.println("ECFMainFrame.askBeforeClosing: end");
	} // askBeforeClosing

	/** Ask before creating a new model because of a "New" command or
	* an "Open XML..." command. If the model was changed, give the user
	* the opportunity to save the the old model or reject = cancel
	* saving.
	* @param optionType = Type of dialog: JOptionPane.YES_NO_CANCEL_OPTION,
	* @param szReason = Reason for asking
	* @param Dialog button pressed: JOptionPane.YES_OPTION,
	*        JOptionPane.NO_OPTION or JOptionPane.CANCEL_OPTION.
	*/
    private int askBeforeNewModel(int optionType, String szReason) {
		int option = JOptionPane.YES_OPTION; // Default answer
		if (ECFModel.fModelWasChanged==true) {
			String szMsg = "Model was changed, save before " + szReason + " ?";
			int messageType = JOptionPane.WARNING_MESSAGE;
			JOptionPane pane = new JOptionPane(szMsg,messageType,optionType);
			JDialog dialog = pane.createDialog(this,"Save model?");
			dialog.show();
			Object selectedValue = pane.getValue();
			if (selectedValue != null) {
				// Dialog was not closed by cross in upper left corner
				option = ((Integer)selectedValue).intValue();
			   	if(option == JOptionPane.YES_OPTION ) {
					int returnVal = saveXMLCode();
					if (returnVal==JFileChooser.CANCEL_OPTION) option = JOptionPane.CANCEL_OPTION;
				} // option
			} // selectedValue
		}// if fModelWasChanged
		return option;
//		System.out.println("ECFMainFrame.askBeforeNewModel: end");
	} // askBeforeNewModel

	/** Get the graphs panel.
	* @return The graphs panel. */
	public GraphsPanel getGraphsPanel() { return panelGraphs; }

	/** Issue a calculator warning.
	* @param fWarn Indication if equation system or calculations are incorrect,
	* true if warning should be shown, false else. */
	public void showCalculatorWarning(boolean fWarn) {
		panelAlert.showCalculatorWarning(fWarn);
	} // showCalculatorWarning

	protected JMenuBar createMenuBar() {
        JMenuBar menuBar = new JMenuBar();

		// File menu
        JMenu menuFile = new JMenu("File");
        menuFile.setMnemonic(KeyEvent.VK_F);

        //   New menu item
        JMenuItem menuItemNewModel = new JMenuItem("New");
        menuItemNewModel.setMnemonic(KeyEvent.VK_N);
        menuItemNewModel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				int option =askBeforeNewModel(JOptionPane.YES_NO_CANCEL_OPTION,"creating new model");
				if (option==JOptionPane.YES_OPTION || option==JOptionPane.NO_OPTION) {
                	ECF.startNewModel();
					panelGraphs.createLayerPanels(ECF.getNoOfLayers());
					setTitle("ECF Version " + ECF.szVersionProgram + "  " + ECF.szDate);
					panelTimeScroll.initSettings();
				} // if option
                repaint();
            } // public void
        });
		menuFile.add(menuItemNewModel);

        //   Open XML coded model menu item
        menuItemOpenXML = new JMenuItem("Open XML...",KeyEvent.VK_L);
        menuItemOpenXML.addActionListener(this);
        menuFile.add(menuItemOpenXML);

        //   Save XML coded model menu item
        menuItemSaveXML = new JMenuItem("Save XML...",KeyEvent.VK_S);
        menuItemSaveXML.addActionListener(this);
        menuFile.add(menuItemSaveXML);

        // Print menu
        JMenu menuPrint = new JMenu("Print");
        menuPrint.setMnemonic(KeyEvent.VK_P);
        // Print to printer
        menuItemPrintToPrinter = new JMenuItem("to Printer, with headings...",KeyEvent.VK_P);
        menuItemPrintToPrinter.addActionListener(this);
		menuPrint.add(menuItemPrintToPrinter);
        // Print to printer with no headings
        menuItemPrintToPrinterNoHeading = new JMenuItem("to Printer, no headings...",KeyEvent.VK_P);
        menuItemPrintToPrinterNoHeading.addActionListener(this);
		menuPrint.add(menuItemPrintToPrinterNoHeading);
        // Print to file
        menuItemPrintToFile = new JMenuItem("to File...",KeyEvent.VK_F);
        menuItemPrintToFile.addActionListener(this);
		menuPrint.add(menuItemPrintToFile);
		// Add to File->Print menu with submenus
        menuFile.add(menuPrint);

      	// Export menu
        JMenu menuExport = new JMenu("Export");
        menuExport.setMnemonic(KeyEvent.VK_E);
        // Text
        menuItemExportText = new JMenuItem("as Text...",KeyEvent.VK_T);
        menuItemExportText.addActionListener(this);
		menuExport.add(menuItemExportText);
        // Excel
        menuItemExportExcel = new JMenuItem("to Excel...",KeyEvent.VK_E);
        menuItemExportExcel.addActionListener(this);
		menuExport.add(menuItemExportExcel);
        // Matlab
        menuItemExportMatlab = new JMenuItem("to Matlab...",KeyEvent.VK_M);
        menuItemExportMatlab.addActionListener(this);
		menuExport.add(menuItemExportMatlab);
        // Maple
        menuItemExportMaple = new JMenuItem("to Maple...",KeyEvent.VK_P);
        menuItemExportMaple.addActionListener(this);
		menuExport.add(menuItemExportMaple);
		// Add to File->Export menu with submenus
		menuFile.add(menuExport);

        //   Exit menu
        menuItemExit = new JMenuItem("Exit",KeyEvent.VK_X);
		menuItemExit.addActionListener(this);
        menuFile.add(menuItemExit);

		// Edit menu
        JMenu menuEdit = new JMenu("Edit");
        menuEdit.setMnemonic(KeyEvent.VK_E);

         //   Time axis menu item
        JMenuItem menuitemTimeAxis = new JMenuItem("Time axis...",KeyEvent.VK_T);
        menuitemTimeAxis.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				try {
					new TimeAxisDialog();
					panelTimeScroll.initSettings();
				} catch(Exception ex) {
					System.out.println(ex);
				}
			} // actionPerformed
        });
		menuEdit.add(menuitemTimeAxis);

        //   Units menu item
        JMenuItem menuitemUnits = new JMenuItem("Units...",KeyEvent.VK_U);
        menuitemUnits.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				UnitsDialog dialog = new UnitsDialog();
				Calculator.calculate();
				panelGraphs.repaintAll();
				repaint();
            }
        });
        menuEdit.add(menuitemUnits);

        //   Edit layer menu item
        JMenuItem menuEditLayer = new JMenuItem("Layer...",KeyEvent.VK_L);
        menuEditLayer.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                LayerFrame frame = (LayerFrame)panelGraphs.getSelectedFrame();
                int ilayer = 0;
                if (frame!=null) ilayer = frame.getIndex();
				LayerDialog dialog = new LayerDialog(mainframe,ECF.getLayer(ilayer),new Point(getX()+20,getY()+20));
				panelGraphs.repaintAll(); mainframe.repaint();
            }
        });
        menuEdit.add(menuEditLayer);

       //   Add a new layer menu item
        JMenuItem menuItemNewLayer = new JMenuItem("Add layer",KeyEvent.VK_A);
        menuItemNewLayer.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ECF.addLayer();
                panelGraphs.createFrame();
                panelGraphs.tileFrames(); // Tile in previous direction
            }
        });
        menuEdit.add(menuItemNewLayer);

        //   Clear variable data menu item
        JMenuItem menuitemClearData = new JMenuItem("Clear data...",KeyEvent.VK_C);
        menuitemClearData.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				new ClearDataDialog();
				panelTimeScroll.initSettings();
                repaint();
            }
        });
        menuEdit.add(menuitemClearData);

        //   Copy variable data menu item
        JMenuItem menuitemCopyData = new JMenuItem("Copy data...",KeyEvent.VK_P);
        menuitemCopyData.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				new CopyDataDialog();
				panelTimeScroll.initSettings();
                repaint();
            }
        });
        menuEdit.add(menuitemCopyData);

		// View menu
        JMenu menuView = new JMenu("View");
        menuView.setMnemonic(KeyEvent.VK_V);

		// Tile menu item
        JMenuItem menuItemTileV = new JMenuItem("Tile vertical",KeyEvent.VK_T);
        menuItemTileV.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                panelGraphs.tileFrames(GraphsPanel.TILE_VERTICAL);
            }
        });
        menuView.add(menuItemTileV);

		// Tile menu item
        JMenuItem menuItemTileH = new JMenuItem("Tile horizontal",KeyEvent.VK_Z);
        menuItemTileH.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                panelGraphs.tileFrames(GraphsPanel.TILE_HORIZONTAL);
            }
        });
        menuView.add(menuItemTileH);

		// Calculator menu item
        JMenuItem menuItemCalculator = new JMenuItem("Calculator...",KeyEvent.VK_C);
        menuItemCalculator.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ECF.calculator.show();
            }
        });
        menuView.add(menuItemCalculator);

        // Hide/show FlowConnectors menu item
        JMenuItem menuHideShowConn = new JMenuItem("Hide/Show connectors",KeyEvent.VK_H);
        menuHideShowConn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				System.out.println("ECFMainFrame.menuHideShowConn:");
				FlowConnector.hideshow();
				mainframe.repaint();
            }
        });
        menuView.add(menuHideShowConn);

        // Show calculated flows or difference between calculated-measured flows
        // menu item
        JMenuItem menuCalcDiff = new JMenuItem("Calculated/Difference flows",KeyEvent.VK_D);
        menuCalcDiff.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				Flow.showCalcMeasFlowDifference();
				mainframe.repaint();
            }
        });
        menuView.add(menuCalcDiff);

        // Show flow numbers menu item
        JMenuItem menuShowFlowNumbers = new JMenuItem("Hide/Show flow numbers",KeyEvent.VK_N);
        menuShowFlowNumbers.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				Flow.showFlowNumbers();
				mainframe.repaint();
            }
        });
        menuView.add(menuShowFlowNumbers);

        menuBar.add(menuFile);
        menuBar.add(menuEdit);
        menuBar.add(menuView);
        return menuBar;
    } // createMenuBar

/*    private class importModelActionListener implements ActionListener {
    	public void actionPerformed(ActionEvent e) {
			// Import ???
    	} // actionPerformed
    } // importModelActionListener
*/
	/** Check if model is consistent with calculator flows.*/
	private void checkModelConsistencyWhenLoading() {
		// Check model consistency
//		int cflowFromModel = ECF.modelTop.getFlows().size();
		int cflowFromModel = ECF.getFlows().size();
		int cflowCalculator = Calculator.getNumberOfFlows();
		if (cflowFromModel!=cflowCalculator) {
			String szMsg = "Number of flows from model " + cflowFromModel +
				"  differs from calculator flows " + cflowCalculator + "\n" +
				"Model file is inconsistent and equation system cannot be properly solved!\n";
			JOptionPane.showMessageDialog(ECF.frameMain,szMsg);
		} // if
	} // checkModelConsistency

	/** Check if model is consistent with calculator flows.
	* @return true if consistent, else false. */
	private boolean checkModelConsistencyWhenSaving() {
		// Check model consistency
//		int cflowFromModel = ECF.modelTop.getFlows().size();
		int cflowFromModel = ECF.getFlows().size();
		int cflowCalculator = Calculator.getNumberOfFlows();
		if (cflowFromModel!=cflowCalculator) {
			String szMsg = "Number of flows from model " + cflowFromModel +
				"  differs from calculator flows " + cflowCalculator + "\n" +
				"Model file is inconsistent due to an unknown program error.\n" +
				"Take care not to overwrite a good model !\n" +
				"Click OK to continue to the save dialog, Cancel to abort.";
			Object[] options = { "OK", "Cancel" };
			int selectedValue = JOptionPane.showOptionDialog(ECF.frameMain,
                                   szMsg,"Warning before saving a bad model",
                                   JOptionPane.YES_NO_OPTION,
                                   JOptionPane.WARNING_MESSAGE,
                                   null,options,options[1]);
			if (selectedValue!=JOptionPane.OK_OPTION) return false;
		} // if
		return true;
	} // checkModelConsistency

	/** Save the model as xml code.
	* @return Dialog button pressed: JFileChooser.CANCEL_OPTION ( default) or
	*         JFileChooser.APPROVE_OPTION. */
	private int saveXMLCode() {
		int returnVal = JFileChooser.CANCEL_OPTION;
		if (checkModelConsistencyWhenSaving()==false) return returnVal;
		// List model components to command window
   	    String szFileName = "ECFModel.xml";
		String szPath = System.getProperty("user.dir");
		returnVal = chooserXML.showSaveDialog(mainframe);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				szFileName = chooserXML.getSelectedFile().getName();
				szPath = chooserXML.getSelectedFile().getAbsolutePath();
				FileWriter filewr = new FileWriter(szPath);
				String szXML = "<ECF>\n" + ECF.listXML("  ") + "</ECF>\n";
				filewr.write(szXML);
				filewr.close();
				szPathTitle = szPath;
				int szPathLength = szPathTitle.length();
				if (szPathLength>70) szPathTitle = szPath.substring(0,15) + " ... " +
					szPath.substring(szPathLength-55,szPathLength);
				ECFModel.fModelWasChanged=false;
				setTitleMainFrame();
				repaint();
			} catch(IOException excIO) {
				String szMsg = "Could not save to file  '" + szFileName +
					" '\n" + excIO.getMessage();
				JOptionPane.showMessageDialog(mainframe,szMsg);
			} catch(Exception exc) {
				String szMsg = "Error when saving to file  '" + szFileName +
					" '\n" + exc.getMessage();
				JOptionPane.showMessageDialog(mainframe,szMsg);
			} // try/catch
		} // if
		return returnVal;
	} // saveXMLCode

	/** List XML code.
	* @return XML code. */
	public String listXML(String szBlanks) {
		String szXML = "";
		szXML += "<panelGraphs>" + panelGraphs.listXML("") + "</panelGraphs>";
		return szXML;
	} // listXML

	private void loadXMLCode() {
//		System.out.println("ECFMainFrame.loadXMLCode: 1");

		String szFileName = "ECFModel.xml";
		String szPath = System.getProperty("user.dir");
		int crow = 0;
		// Open XML code for model
		int returnVal = chooserXML.showOpenDialog(mainframe);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
//		System.out.println("ECFMainFrame.loadXMLCode: 2");
			try {
				// Open file and read content
				szFileName = chooserXML.getSelectedFile().getName();
				szPath = chooserXML.getSelectedFile().getAbsolutePath();
				// This is an attempt to add a progress bar.
				File file = new File(szPath);
				System.out.println("ECFMainFrame.loadXMLCode: " + szFileName +" size = "
				+ file.length()/1024 + " kB");
				BasicProgressBarUI progressbar = new BasicProgressBarUI();
				Date dateStart = new Date();
				long timeStart = dateStart.getTime();
				// Read file
				BufferedReader buffreader = new BufferedReader(new FileReader(szPath));
				String szXMLCode = "";
				String szLine = buffreader.readLine();
				while (szLine!=null) {
					szXMLCode += szLine + "\n";
					szLine = buffreader.readLine();
					crow++;
				} // while
				buffreader.close();
//				System.out.println("ECFMainFrame.loadXMLCode: 3");

				// Decode XML code
				decodeXML(szXMLCode);
				Date dateEnd = new Date();
				long timeEnd = dateEnd.getTime();
				double timeLapse = (timeEnd-timeStart);
				System.out.println("ECFMainFrame.loadXMLCode: Loading time = " + timeLapse + " ms\n");

				// Set up graphs panels
				int cpanel = ECF.getNoOfLayers();
				try {
					// Remove all LayerFrames
					panelGraphs.createLayerPanels(cpanel);
					szPathTitle = szPath;
					int szPathLength = szPathTitle.length();
					if (szPathLength>70) szPathTitle = szPath.substring(0,15) + " ... " +
						szPath.substring(szPathLength-55,szPathLength);
					setTitleMainFrame();
					panelAlert.update();
					repaint();
					ECFModel.fModelWasChanged=false;
				} catch(Exception e) {
					System.out.println(e);
				}
			} catch(FileNotFoundException excFNF) {
				String szMsg = "Could not find file '" + szFileName + " '\n" + excFNF.getMessage();
				JOptionPane.showMessageDialog(mainframe,szMsg);
			} catch(XMLParserException excXML) {
				String szMsg = "File  '" + szFileName + "'  has not a correct XML format\n" +
					excXML.getMessage();
				JOptionPane.showMessageDialog(mainframe,szMsg);
			} catch(Exception exc) {
				String szMsg = "Error when opening file '" + szFileName + " '\n" + exc.getMessage();
				JOptionPane.showMessageDialog(mainframe,szMsg);
			} // try/catch
//			System.out.println("ECFMainFrame.loadXMLCode: 5");
//			checkModelConsistencyWhenLoading();
		} // if
//		System.out.println("ECFMainFrame.loadXMLCode: end");
	} // loadXMLCode

	/** Set title of main frame.*/
	public void setTitleMainFrame() {
		String szTitle = "ECF Version " + ECF.szVersionProgram + "  " + szPathTitle;
		if (ECFModel.fModelWasChanged) szTitle += " *";
		setTitle(szTitle);
	} // setTitleMainFrame

	/** Decode interface and model from XML code.
	* @param szXMLCode The xml description of the interface and the model. */
	public void decodeXML(String szXMLCode) throws XMLParserException {
		XMLParser xml = null;
		if (szXMLCode.length()==0) {
			JOptionPane.showMessageDialog(this,
   				"Your model file is empty. \n",
   				"Empty file",
   				JOptionPane.ERROR_MESSAGE);
		} else {
			xml = new XMLParser(szXMLCode);
			String szTag = xml.tag();
			String szSub = xml.sub();
			if (!szTag.equals("ECF"))  {
				JOptionPane.showMessageDialog(this,
    				"You are not opening an ECF model file. \n",
	   				"Wrong file",
    				JOptionPane.ERROR_MESSAGE);
			} else {// if/else
				ECF.decodeXML(szSub);
			}
		} // if/else
	} // decodeXML

	/** Decode interface and model from XML code.
	* @param szXMLCode The xml description of the interface and the model. */
	public void decodeXMLFrame(String szXMLCode) throws XMLParserException {
		XMLParser xml = null;
		while (szXMLCode.length()>0) {
			xml = new XMLParser(szXMLCode);
			String szTag = xml.tag();
			String szSub = xml.sub();
			if (szTag.equals("panelGraphs"))  {
				panelGraphs.decodeXML(szSub);
			}
			szXMLCode = xml.remainder();
		} // while
	} // decodeXML

	public void actionPerformed(ActionEvent e) {
		if (e.getSource()==menuItemSaveXML) {
			saveXMLCode();
		} else if (e.getSource()==menuItemOpenXML) {
			int option = askBeforeNewModel(JOptionPane.YES_NO_CANCEL_OPTION,"loading new model");
	        if (option==JOptionPane.YES_OPTION || option==JOptionPane.NO_OPTION) {
	        	ECF.startNewModel();
				loadXMLCode();
				panelTimeScroll.initSettings();
			} // if option
		} else if (e.getSource()==menuItemPrintToPrinter) {
			ECFPrinterJob printerJob = new ECFPrinterJob(true);
		} else if (e.getSource()==menuItemPrintToPrinterNoHeading) {
			ECFPrinterJob printerJob = new ECFPrinterJob(false);
		} else if (e.getSource()==menuItemPrintToFile) {
			new RenderedImage();
		} else if (e.getSource()==menuItemExportText) {
			ECF.calculator.exportText(getFileName());
		} else if (e.getSource()==menuItemExportExcel) {
			ECF.calculator.exportExcel(getFileName());
		} else if (e.getSource()==menuItemExportMatlab) {
			ECF.calculator.exportMatlab(getFileName());
		} else if (e.getSource()==menuItemExportMaple) {
			ECF.calculator.exportMaple(getFileName());
		} else if (e.getSource()==menuItemExit) {
	        askBeforeClosing(JOptionPane.YES_NO_CANCEL_OPTION);
		} else if (e.getSource()==comboxZoom) {
			int iscale = comboxZoom.getSelectedIndex();
			ECFModel.scaleDisplay = vrscale[iscale];
			repaint();
		} // if/else e.getSource()==menuItemXXX
	} // actionPerformed

} // class ECFMainFrame


