import java.awt.*;
import java.util.*;

/** Utility methods. */
public class ECFUtil {

	/** Center frame on screen.
	* @param frame The frame to be centered.
	*/
    public static void center(Window frame) {
		// EFFECTS: centers this frame within the screen
		Dimension sizeScreen = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sizeFrame = frame.getSize();
		int x = (sizeScreen.width - sizeFrame.width)/2;
		int y = (sizeScreen.height - sizeFrame.height)/2;
		frame.setLocation(x,y);
	} // center()

	/** Remove line feeds from a string.
	* @param sz The original string with line feeds.
	* @return A string without the line feeds. */
	public static String removeLineFeeds(String sz) {
		int ichar = sz.indexOf('\n');
		while (ichar>0) {
			sz = sz.substring(0,ichar) + sz.substring(ichar+1);
			ichar = sz.indexOf('\n');
		} // while
		return sz;
	} // removeLineFeeds

	/** Get current date and time as yyyy-mm-dd hh:mm,
	* example 2005-12-18 19:03.
	* @return A string with the current time. */
	public static String getCurrentTime() {
			GregorianCalendar calendar = new GregorianCalendar();
			Date timeNow = new Date();
			calendar.setTime(timeNow);
			String szTimeNow = calendar.get(Calendar.YEAR) + "-" +
				padZeroes(1+calendar.get(Calendar.MONTH),2) + "-" +
				padZeroes(calendar.get(Calendar.DATE),2) + "  " +
				calendar.get(Calendar.HOUR_OF_DAY) + ":" +
				padZeroes(calendar.get(Calendar.MINUTE),2);
			return szTimeNow;
	} // getCurrentTime

	/** Get the file name without extension.
	* @return The file name up to the last period character ".".
	*/
	public static String getFileNameWithoutExtension(String szFullName) {
		int icharPeriod = szFullName.indexOf(".");
		String szShortName = szFullName.substring(0,icharPeriod);
		return szShortName;
	} // getFileNameNoExtension

	/** Pad with leading characters.
	* @param num A integer number to be converted to a string with cchar characters.
	* @param cchar Number of characters to be returned.
	* @param character The character to be padded
	* @return A string of length cchar with leading zeros. */
	public static String padCharacters(int num, int cchar, char character) {
		String sz = Integer.toString(num);
		while (sz.length()<cchar) sz = character + sz;
		return sz;
	} // padZeroes

	/** Pad with leading zeros.
	* @param num A integer number to be converted to a string with cchar characters.
	* @param cchar Number of characters to be returned.
	* @return A string of length cchar with leading zeros. */
	public static String padZeroes(int num, int cchar) {
		String sz = Integer.toString(num);
		while (sz.length()<cchar) sz = "0" + sz;
		return sz;
	} // padZeroes

	public static void main(String[] args) {
		System.out.println("Test getCurrentTime: " + ECFUtil.getCurrentTime());
		System.out.println("Test padCharacters:  " + ECFUtil.padCharacters(5,3,' '));
		System.out.println("Test padCharacters:  " + ECFUtil.padCharacters(17,3,'0'));
	} // main

} // ECFUtil