import javax.swing.*;     // JFileChooser
import java.io.*;         // File
import java.util.*;		  // Date
import java.text.*;       // DateFormat

/** Exports model data to Excel file.*/
public class ExcelExporter extends Exporter {

	public static String szCommandStart = "";
	public static String szCommentSign = "% ";

	/** Creates a ExcelExporter for writing model data to file.
	* @param frame The frame to which the dialog belongs.
	* @param szNameXMLFile The name of the XML model file.
	* @param szExtension File extension of exported file. */
	public ExcelExporter(String szNameXMLFile, String szExtension) {
		super(szNameXMLFile,szExtension);
	} // ExcelExporter

	/** Returns the comment sign for the specific file format,
	* should be replaced in subclass. */
//	protected String commentSign() { return "% ";}

	/** Lists the model data for the specific file format. */
/*	protected String listData() {
		String szExport = "";
		// Heading
		szExport += listHeading();
		// Time histories
		szExport += listTimeHistories();
		return szExport;
	} // listData
*/

	/** List nothing at selected time. */
	public String listAtSelectedTime() { return ""; }

	/** List the heading.
	* @return A string with the heading.*/
	public String listHeading() {
		String szExport = "";
		// Heading
		szExport += "\n" + szCommentSign + "Time histories;\n" +
			szCommentSign + "=======================================;\n\n";
		return szExport;
	} // listHeading

	/** List the time histories of variables.
	* @return A string listing time histories.*/
	public String listTimeHistories() {
		String szExport = "";
		// Time axis
		if (timeaxis!=null)	szExport += timeaxis.export();
		// Data vectors
		szExport += listFlowTimeHistories();
		szExport += listFlowRatioTimeHistories();
		szExport += listPriceTimeHistories();
		return szExport;
	} // listTimeHistories

	/** List flow time histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each flow line. */
	private String listFlowTimeHistories() {
		// Flows as a function of time
		String szFlows = "";
		if (vrflow==null)	return szFlows;
		Iterator iter = vrflow.iterator();
		while (iter.hasNext()) {
			szFlows += ((Flow)iter.next()).export();
		} // while
		return szFlows;
	} // listFlowTimeHistories

	/** List flow ratio time histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each flow ratio line. */
	private String listFlowRatioTimeHistories() {
		// Flows as a function of time
		String szFlowRatios = "";
		if (vrflowratio==null)	return szFlowRatios;
		Iterator iter = vrflowratio.iterator();
		while (iter.hasNext()) {
			szFlowRatios += ((FlowRatio)iter.next()).export();
		} // while
		return szFlowRatios;
	} // listFlowTimeHistories

	/** List price histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each price line. */
	private String listPriceTimeHistories() {
		// Flows as a function of time
		String szPrices = "";
		if (vrprice==null)	return szPrices;
		Iterator iter = vrprice.iterator();
		while (iter.hasNext()) {
			szPrices += ((Price)iter.next()).export();
		} // while
		return szPrices;
	} // listPriceTimeHistories

} // ExcelExporter