import java.awt.*;        // Graphics2D
import javax.swing.*;     // JOptionPane

/** A explanatorylabel that can be positioned in any place in a graphics context,
* used for explanations in the model diagram area. */
public class ExplanatoryLabel extends GraphLabel {

	/** Index of the layer in which the label is displayed. */
	protected int ilayer = 0;
	/** X-coordinate of label before translating. */
	protected int xStart;
	/** Y-coordinate of labelbefore translating. */
	protected int yStart;

	/** Creates a explanatorylabel label.
	* @param szText The text of the label.
	*/
	public ExplanatoryLabel(String szText) {
		super(szText,0,0,Color.white);
	} // GraphLabel

	/** Creates a explanatorylabel label.
	* @param szText The text of the label.
	* @param x X-coordinate, pixels, for the lower left corner.
	* @param y Y-coordinate, pixels, for the lower left corner.
	*/
	public ExplanatoryLabel(String szText, int ilayer ,int x, int y) {
		super(szText,x,y,Color.white);
		this.ilayer = ilayer;
	} // ExplanatoryLabel

	/**
	* Checks if the <code>ExplanatoryLabel</code> belongs to layer <code>ilayer</code>
	* and contains the coordinates (x,y).
	* Sets label as selected.
	* @return true if contained, false otherwise.
	*/
	public boolean contains(int ilayer,int x,int y) {
		if (this.ilayer==ilayer)  {
			if (contains(x,y))  {
				return true;
			} // if fLabelSelected
		} // ilayer
		return false;
	} // contains

	/**
	* Decode label from XML code.
	* @param szXMLCode The xml description of the label.
	*/
	public void decodeXML(String szXMLCode) {
		XMLParser xml = null;
		// Decode XML code
		try {
			while (szXMLCode.length()>0) {
				xml = new XMLParser(szXMLCode);
				String szTag = xml.tag();
				String szSub = xml.sub();
				if (szTag.equals("ilayer")) ilayer = Integer.parseInt(szSub);
				if (szTag.equals("GraphLabel")) super.decodeXML(szSub);
				szXMLCode = xml.remainder();
			} // while
		} catch(Exception e) {
			JOptionPane.showMessageDialog(ECF.frameMain,
				"Could not decode XML model file: \n" + e +
				"\nMail your model file to olert@swipnet.se",
				"ECFModel.decodeXML",JOptionPane.WARNING_MESSAGE);
			System.out.println("ExplanatoryLabel.decodeXML: \n" + e);
			System.out.println("ExplanatoryLabel.decodeXML: \n" + szXMLCode);
			System.exit(1);
		} // try/catch
	} // decodeXML

	/** List XML code for this model.
	* @return XML code. */
	public String listXML(String szBlanks1) {
		String szBlanks = ""; // To give one compact line
		String szXML = "";
		szXML += szBlanks +"<ilayer>" + ilayer + "</ilayer>";
		szXML += szBlanks +"<GraphLabel>" + super.listXML("") + "</GraphLabel>";
		return szXML;
	} // listXML

	/**
	* Translate the label by (dx,dy)
	* @param dx translation in x direction.
	* @param dy translation in y direction.
	*/
	public void translate(int dx,int dy) {
		setPos(xStart+dx, yStart+dy);
	} // translate

	/** Open a dialog for setting properties of the label. */
	public void openDialog() {
		new LabelDialog(ECF.frameMain,this);
	} // openDialog

	/** Paints the label.
	* @param g2 graphics context.
	* @param ilayer the index of the model layer beeing drawn.
	*/
	public void paint(Graphics2D g2, int ilayer) {
		if (this.ilayer!=ilayer) return;
		super.paint(g2);
	} // Paint

	/** Set label as deselected. */
	public void setDeselected() { setBackgroundDefault(Color.white); }

	/**
	* Set start position (before translating) of the label.
	*/
	public void setStartPos() {
		xStart = x;
		yStart = y;
	} // setStartPos

	/** Return a string description.
	* @return A string describing the label. */
	public String toString() { return "ExplanatoryLabel: " + szText +
		" ilayer=" + ilayer + " x=" + x + " y=" + y; }

} // ExplanatoryLabel