/**
* A pair of flow and coefficient.
* Used for building linear equations for flows of the type:<br>
* coeff1*flowvalue1 + coeff2*flowvalue2 + ... + coeffN*flowvalueN = 0
*/

public class FlowCoeff {

	/** The flow to which the coefficient belongs.*/
	public Flow flow;
	/** The coefficient of the flow*/
	public double coeff;

	/**
	* Creates a pair of <code>flow</code> and cofficient <code>coeff</code>.
	* @param flow The flow.
	* @param coeff The coefficient
	*/
	public FlowCoeff(Flow flow, double coeff) {
		this.flow = flow;
		this.coeff = coeff;
	} // FlowCoeff

	/**
	* Creates an empty pair of <code>flow</code> and cofficient <code>coeff</code>.
	*/
	public FlowCoeff() {
		flow = null;
		coeff = 0.0;
	} // FlowCoeff

	/**
	* Decode flow coefficent from XML code.
	* @param szXMLCode The xml description of the flow coefficient.
	*/
	public void decodeXML(String szXMLCode) {
		XMLParser xml = null;
		// Decode XML code
		try {
			while (szXMLCode.length()>0) {
				xml = new XMLParser(szXMLCode);
				String szTag = xml.tag();
				String szSub = xml.sub();
				if (szTag.equals("flowindex")) {
					int iflow = Integer.parseInt(szSub);
					flow = Calculator.getFlow(iflow);
				} // if
				if (szTag.equals("coeff")) coeff = Double.parseDouble(szSub);
				szXMLCode = xml.remainder();
			} // while
		} catch(Exception e) {
			System.out.println(e);
		} // try/catch
	} // decodeXML

	/** Get the flow. Can also be retrieved by <code>flowcoeff.flow</code>
	* @return The flow
	*/
	public Flow getFlow() { return flow; }

	/** Get the coeff. Can also be retrieved by <code>flowcoeff.coeff</code>
	* @return The coeff
	*/
	public double getCoeff() { return coeff; }

	/** List XML code for the splitter.
	* @return XML code. */
	public String listXML(String szBlanks) {
		String szXML = "";
		szXML += szBlanks +"<flowindex>"+flow.getIndexOfFlow()+"</flowindex>\n";
		szXML += szBlanks +"<coeff>"+coeff+"</coeff>\n";
		return szXML;
	} // listXML

	/** Makes a string describing the flow-coefficient pair.*/
	public String toString() {
		return "FlowCoeff: flow=" + flow.getName() + " coeff=" + coeff;
	} // toString

} // FlowCoeff