import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;

/**
* A <code>FlowPathOfFrame</code> connects a node to the frame of a submodel.
* Resources flow in the direction indicated by the arrow.
* Resources may be monetary or real. Many flows may be associated to one
* flowpath. The flows describe the kind of resource and the quantity.
* <p>
* Each flowpath belongs to one model layer. It has two end points with connectors
* that can be dragged by the mouse. They can be connected to a node if dragged
* over that node and then released. The connection is done from the node side
* via <A href="NodeConnectors.html">NodeConnectors</A>.
* <p>
* The <code>FlowPath</code> class has only metods for disconnection flows: <br>
* - <code>disconnectThis()</code> disconnects both ends (before deleting the flow path) <br>
* - <code>translate(dx,dy)</code> disconnects the translated end via the
* <A href="FlowConnector.html">FlowConnector</A> class.
*/
public class FlowPathOfFrame extends FlowPath {

	/** The twin flow connector of the flow path. */
	TwinFlowConnector twinflowconn;
	/** Index of flow path. */
	protected int iflowpath = 0;

	public FlowPathOfFrame(TwinFlowConnector twinflowconn, FlowPathHolder model,
			int ilayer,int x1,int y1,int x2,int y2, Flow flow) {
		super(model,ilayer,x1,y1,x2,y2,flow);
		this.twinflowconn = twinflowconn;
	} // FlowPathOfFrame

	/** Get the index of the flow path in the parent model. */
	public int getFlowPathIndex() {
		return iflowpath;
		} // getFlowPathIndex

	/** Get the index of the flow path in the parent model. */
	public void setFlowPathIndex(int iflowpath) {
		this.iflowpath = iflowpath;
		} // setFlowPathIndex

	/** Get side of frame to which connector is connected.
	* @return side of frame.*/
	public int getSideOfFrame() {
		try {
			return twinflowconn.getSideOfFrame();
		} catch (Exception e) {
			System.out.println("FlowPathOfFrame.getSideOfFrame:" + e);
		} // try/catch
		return 0;
	} // getSideOfFrame

	/** List XML code for this flow path of frame.
	* @return XML code. */
	public String listXML(String szBlanks) {
		String szXML = "";
/*		szXML += szBlanks +"<iflowpath>"+ getFlowPathIndex()+
				"</iflowpath>\n";
		szXML += szBlanks +"<isideOfFrame>"+ getSideOfFrame()+
				"</isideOfFrame>\n";
		szXML += szBlanks +"<twinflowconn>\n";
		szXML += twinflowconn.listXML("  " + szBlanks);
		szXML += szBlanks +"</twinflowconn>\n";
*/		szXML += "<iflowpath>"+ getFlowPathIndex()+"</iflowpath>";
		szXML += "<isideOfFrame>"+ getSideOfFrame()+"</isideOfFrame>";
		szXML += "<twinflowconn>";
		szXML += twinflowconn.listXML("");
		szXML += "</twinflowconn>";
		return szXML;
	} // listXML

} // FlowPathOfFrame