import java.util.*;         // Vector
import javax.swing.table.*; // DefaultTableModel
import javax.swing.event.*; // TableModelEvent
import java.text.ParseException;

/**
* The FlowTableModel stores the values
* of a <A href="Flow.html">Flow</A> to be displayed by a
* FlowDialog.
*/

public class FlowTableModel extends DefaultTableModel  implements TableModelListener {

	protected FlowDialog dialog;
	protected Flow flow;
	protected Vector vrnameColumn = new Vector();
	protected Vector vrData = new Vector();
	protected Vector vrVariableCalculated = new Vector();
	protected Vector vrVariablePrescribed = new Vector();
	protected double vrvalueCalculated[];
	protected double vrvaluePrescribed[];

	/**
	* Creates a FlowTableModel that that is used by a FlowDialog to
	* display an edit the values of a Flow.
	*/

    public FlowTableModel(FlowDialog dialog, Flow flow) {

		this.dialog = dialog;
		this.flow = flow;

		// Set row labels
		vrnameColumn.add("Time");
		vrVariableCalculated.add("Calculated");
		vrVariablePrescribed.add("Prescribed");

		int ccol = flow.size();
		vrvaluePrescribed = new double[ccol];
		Vector vrtimeInc = flow.getTimeAxis().getTimeIncrements();
		double[][] mxvalueDisplay = flow.getDisplayValues();

		// Set column labels and data
		for (int icol=0; icol<ccol; icol++) {
			TimeIncrement timeinc = (TimeIncrement)vrtimeInc.elementAt(icol);
			String szTime = String.valueOf(timeinc.getStartTime());
			vrnameColumn.add(szTime);
			vrVariableCalculated.add(Quantity.formatDouble(mxvalueDisplay[1][icol]));
			vrVariablePrescribed.add(Quantity.formatDouble(mxvalueDisplay[0][icol]));
		} // for icol

		// Set up tabel data
		vrData.add(vrVariablePrescribed);
		vrData.add(vrVariableCalculated);
		setDataVector(vrData,vrnameColumn);
		// Add listener
		addTableModelListener(this);

	} // FlowTableModel

    /*
     * JTable uses this method to determine the default renderer/
     * editor for each cell.
     */
    public Class getColumnClass(int icol) {
		try {
        	return getValueAt(0, icol).getClass();
		} catch(Exception e) {
			System.out.println(e);
			return null;
		} // try
    } // getColumnClass

    /*
     * Don't need to implement this method unless your table's
     * editable.
     */
    public boolean isCellEditable(int irow, int icol) {
        //Note that the data/cell address is constant,
        //no matter where the cell appears onscreen.
        if (icol==0 || irow==1 || !flow.isPrescribed()) {
            return false;
        } else {
            return true;
        }
    } // isCellEditable

	/*
	 * Update the variable
	 * Method of interface TableModelListener
	 */
    public void tableChanged(TableModelEvent e) {
		int cinc = vrnameColumn.size();
		String szCell = "0.0";
		Vector vrtimeInc = flow.getTimeAxis().getTimeIncrements();
		for (int iinc=0; iinc<cinc-1; iinc++) {
			TimeIncrement timeinc = (TimeIncrement)vrtimeInc.elementAt(iinc);
			String szTime = String.valueOf(timeinc.getStartTime());
			try {
				szCell = (String)vrVariablePrescribed.elementAt(iinc+1);
				vrvaluePrescribed[iinc] = Quantity.parseDouble(szCell);
				dialog.showErrorMsg(" ");
			} catch(ParseException pex) {
				dialog.showErrorMsg("\"" + szCell + "\" at time \"" +
					szTime + "\" is not a valid decimal number.");
				// table.setSelectionForeground(Color.red);
				break;
			} catch(Exception ex) {
				dialog.showErrorMsg("Unexpected error while parsing \"" + szCell +
				"\" at time \"" + szTime + "\": " + ex);
				break;
			} // try/catch
		} // for
		try {
		flow.setDisplayValuesPrescribed(vrvaluePrescribed); // Convert to base values in Flow
		dialog.repaint();
		} catch(Exception ex) {
			System.out.println("FlowTableModel.tableChanged: flow=" + ex);
		}
	} // tableChanged

} // FlowTableModel