import java.awt.*;
import java.util.*;
import javax.swing.*; // JDesktopPane, JFrame

public class GraphsPanel extends JDesktopPane {

	/** Tile frames vertically. */
	public static final int TILE_VERTICAL = 1;
	/** Tile frames horizontally. */
	public static final int TILE_HORIZONTAL = 2;

	JFrame frameMain;
	ECFTool tool = null;
	DesktopManager desktopmanager = new DefaultDesktopManager();
	/** Mode for tiling frames. */
	protected int imodeTile = GraphsPanel.TILE_VERTICAL;

	GraphsPanel(JFrame frameMain) {
		this.frameMain = frameMain;
		setDesktopManager(desktopmanager);
		createFrame(); //Create first internal frame
	} // GraphsPanel

	/** Create an additional internal frame. */
    public void createFrame() {
		try {
			LayerFrame frameLayer = new LayerFrame(this);
			frameLayer.setVisible(true); //necessary as of kestrel
			if (tool!=null) frameLayer.install(tool);
    	    add(frameLayer);
    	    try {
    	        frameLayer.setSelected(true);
    	    } catch (java.beans.PropertyVetoException e) {}
		} catch(Exception exc) { System.out.println(exc); }
    } // createFrame

	public JFrame getMainFrame() {return frameMain;}

	private void closeAllInternalFrames() {
		JInternalFrame[] vrframe = getAllFrames();
		for (int iframe=0; iframe<vrframe.length; iframe++) {
			desktopmanager.closeFrame(vrframe[iframe]);
		}
	} // closeAllInternalFrames

	/** Remove all previous panels and create new layer panels.
	* @param cpanel number of panels to be created. */
	public void createLayerPanels(int cpanel) {
		// Remove all LayerFrames
		closeAllInternalFrames();
		removeAll();
		LayerFrame.resetFrameCount();
		// Create new LayerFrames
		for (int ipanel=0; ipanel<cpanel; ipanel++) {
			createFrame();
		} // for
		// Refresh
		frameMain.repaint();
		tileFrames();
	} // clearLayerPanels

	/**
	* Decode graphs panel from XML code.
	* @param szXMLCode The xml description of the graphs panel.
	*/
	public void decodeXML(String szXMLCode) {
		XMLParser xml = null;
		// Decode XML code
		try {
			while (szXMLCode.length()>0) {
				xml = new XMLParser(szXMLCode);
				String szTag = xml.tag();
				String szSub = xml.sub();
				if (szTag.equals("imodeTile"))  {
					imodeTile = Integer.parseInt(szSub);
					tileFrames();
				} // imodeTile
				szXMLCode = xml.remainder();
			} // while
		} catch(Exception e) {
		} // try/catch
	} // decodeXML

	/** Tile internal frames i current direction. */
	public void tileFrames() { tileFrames(imodeTile); }

	/** Tile internal frames in the imodeTile direction.
	* @param imodeTile The tile direction,
	* <code>TILE_VERTICAL</code> or <code>TILE_HORIZONTAL</code>. */
	public void tileFrames(int imodeTile) {
		// Tile frames that are not iconized.
		JInternalFrame[] vrframe = getAllFrames();
		Dimension dimPane = getSize();
		int heightPane = (int)dimPane.getHeight();
		int widthPane = (int)dimPane.getWidth();
		this.imodeTile = imodeTile;

		// Extract not iconized frames
		JInternalFrame[] vrframeOpen = new JInternalFrame[vrframe.length];
		int cframe = 0;
		int heightIcon = 0;
		for (int iframe=0; iframe<vrframe.length; iframe++) {
			JInternalFrame frame = vrframe[iframe];
			if (frame.isIcon()) {
				JInternalFrame.JDesktopIcon icon = frame.getDesktopIcon();
				heightIcon = icon.getHeight();
			}
			else {
				vrframeOpen[iframe] = frame;
				int indexFrameLast = ((LayerFrame)frame).getIndex(); // index when first created
				cframe++;
				// Sort frames according to index when created
				int iframePrev = iframe - 1;  // indices in vector
				int indexFramePrev = 0;       // index when first created
				if (iframePrev>=0) indexFramePrev = ((LayerFrame)vrframeOpen[iframePrev]).getIndex();
				while (iframePrev>=0 && indexFrameLast<indexFramePrev) {
					vrframeOpen[iframePrev+1] = vrframeOpen[iframePrev];
					vrframeOpen[iframePrev] = frame;
					iframePrev--;
					if (iframePrev>=0) indexFramePrev = ((LayerFrame)vrframeOpen[iframePrev]).getIndex();
				} // while
			} // if/else
		} // for

		if (cframe==0) return;

		if (imodeTile==TILE_VERTICAL) {
			// Distribute open frames from left to right
			int widthFrame = widthPane/cframe;
			for (int iframe=0; iframe<cframe; iframe++) {
				vrframeOpen[iframe].setBounds(iframe*widthFrame, 0, widthFrame, heightPane-heightIcon);
			} // for
		} // if vertical

		if (imodeTile==TILE_HORIZONTAL) {
			// Distribute open frames from top to bottom
			int widthFrame = widthPane/cframe;
			int heightFrame = (heightPane-heightIcon)/cframe;
			for (int iframe=0; iframe<cframe; iframe++) {
				vrframeOpen[iframe].setBounds(0, iframe*heightFrame, widthPane, heightFrame);
			} // for
		} // if vertical

	} // tileFrames

    public void install(ECFTool tool) {
		this.tool = tool;
		JInternalFrame[] vrframe = getAllFrames();
		for (int iframe=0; iframe<vrframe.length; iframe++) {
			LayerFrame frame = (LayerFrame)vrframe[iframe];
			if (!(frame==null)) frame.install(tool);
		} // for
	} // install

	/** List XML code for the graphs panel.
	* @return XML code. */
	public String listXML(String szBlanks) {
		String szXML = "";
		// Tile mode
		szXML += "<imodeTile>" + imodeTile + "</imodeTile>";
		return szXML;
	} // listXML

	public void repaintAll() {
		JInternalFrame[] vrcomp = getAllFrames();
		for (int iframe=0; iframe<vrcomp.length; iframe++) {
			LayerFrame frameLayer = (LayerFrame)vrcomp[iframe];
			frameLayer.repaint();
		} // for
	} // repaintAll

} // GraphsPanel