import java.awt.*;       // Dimension, BorderLayout, Toolkit
import javax.swing.*;    // JDialog
import java.awt.event.*; // ActionListener,ItemListener
import java.util.*;      // StringTokenizer

/** The dialog imports properties of a Variable.*/
public class ImportDialog extends JDialog implements
	ActionListener,ItemListener{

	JDialog dialogOwner;
	Variable variable;
	double[] vrvalueNew;
	int cnum = 0;
	JTextField textPasted;
	String szLabelConfirmDefault = "Are these the data that you expected?";
	JLabel lblConfirm;
	JTextField textConfirm;
	JButton btnOK = new JButton("OK");
	JButton btnCancel = new JButton("Cancel");

	/** Creates a dialog for importing variable values.
	* @param owner The owner of the dialog.
	* @param szTitle The title of the dialog.
	* @param variable The variable which values shall be entered.
	*/
//	public ImportDialog(VariableDialog owner, String szTitle, Variable variable) {
	public ImportDialog(JDialog owner, String szTitle, Variable variable) {
		super(owner, szTitle, true);
		dialogOwner = owner;
		this.variable = variable;
		cnum = variable.size();
		vrvalueNew = new double[cnum];
		init();
	} // ImportDialog

	/** Center the dialog on the screen. */
    private void center() {
		// EFFECTS: centers this dialog within the screen
		Dimension sizeScreen = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sizeFrame = getSize();
		int x = (sizeScreen.width - sizeFrame.width)/2;
		int y = (sizeScreen.height - sizeFrame.height)/2;
		setLocation(x,y);
	} // center()

	/** Initializes position and content of the dialog. */
	protected void init() {
		makeContentPane();
		setSize(600,200);
		center();
		show();
	} // init

	protected void makeContentPane() {

		Container pane = getContentPane();
		pane.setLayout(new BorderLayout());

		// Add top panel
		pane.add(makeTopPanel(),BorderLayout.NORTH);
		pane.add(makeDataPanel(),BorderLayout.CENTER);
		pane.add(makeButtonsPanel(),BorderLayout.SOUTH);

	} // makeContentPane

	protected JPanel makeTopPanel() {
		JPanel panel = new JPanel();
		String szHeading = "Name: " + variable.getName();
		Units units = variable.getUnits();
		if (units!=null)
			szHeading += "(" + units.getNameOfDisplayUnit() + ")";
		szHeading += "   Paste or enter " + cnum;
		if (cnum==1) szHeading += " value "; else szHeading += " values ";
		szHeading += "below and press ENTER";
		JLabel lbl = new JLabel(szHeading);
		panel.add(lbl);
		return panel;
	} // makeTopPanel

	protected JScrollPane makeDataPanel() {

		JPanel panel = new JPanel(new GridLayout(2,1));
		JScrollPane scrollpane = new JScrollPane(panel);

		textPasted = new JTextField(200);
		textPasted.setToolTipText("Use 'Home' and 'End' keys to see first and last charaters.");
		textPasted.addActionListener(this);
		panel.add(textPasted);

		textConfirm = new JTextField(200);
		textConfirm.setToolTipText("Press 'Enter' key.");
		textConfirm.setEditable(false);
		panel.add(textConfirm);
		// panel.pack();
		return scrollpane;
	} // makeDataPanel

	protected JPanel makeButtonsPanel() {
		JPanel panel = new JPanel(new GridLayout(2,1));

		JPanel panelLabelConfirm = new JPanel();
		lblConfirm = new JLabel(szLabelConfirmDefault);
		panelLabelConfirm.add(lblConfirm);
		panel.add(panelLabelConfirm);

		JPanel panelButtons = new JPanel();
		btnOK.setEnabled(false);
		btnOK.addActionListener(this);
		panelButtons.add(btnOK);
		btnCancel.addActionListener(this);
		panelButtons.add(btnCancel);
		panel.add(panelButtons);

		return panel;
	} // makeButtonsPanel

	public void actionPerformed(ActionEvent e) {

		// Paste text field
		if (e.getSource()==textPasted) {  // Enter button pressed
			String szPasted = textPasted.getText();
			textPasted.setColumns(szPasted.length());
			parseData(szPasted,cnum,vrvalueNew);

			String szConfirm = "";
			for (int inum=0; inum<cnum; inum++) {
				String szNum = String.valueOf(vrvalueNew[inum]);
				szConfirm += szNum + "  ";
			} // for
			textConfirm.setText(szConfirm);
			textConfirm.setToolTipText("This is how the program understands the text above.");
			textConfirm.setColumns(szConfirm.length());
			btnOK.setEnabled(true);
		} // Paste text field

		// OK button
		if (e.getSource()==btnOK) {
			variable.setDisplayValuesPrescribed(vrvalueNew);
//			dialogOwner.setVaribleMeasured(true);
			dispose();
		} // OK button

		// Cancel button
		if (e.getSource()==btnCancel) {
			dispose();
		} // Cancel button

	} // actionPerformed


	public void itemStateChanged(ItemEvent e) {
//		if (textPasted.getState()) {
			System.out.println("ImportDialog.itemStateChanged: ");
//		} // if
	} // itemStateChanged(ItemEvent e)

	/** Gets the data from the string szData.
	* The string may hold less data than the entries in the variable,
	* then the missing data is appended as zeros (0.0).
	* The string may also hold more data than the entries in the variable,
	* then the excess data is omitted.
	* @param szData A text string that holds the data.
	* @param cnum The number of numbers that shall be extracted from szData. */
	protected void parseData(String szData, int cnum, double[] vrvalueNew) {

		StringTokenizer st = new StringTokenizer(szData);
		TimeAxis timeaxis = ECF.getTimeAxis();
		int ctok = st.countTokens();
		if (ctok!=cnum) {
			if (ctok!=1) lblConfirm.setText(ctok+ " numbers were entered! ");
			else lblConfirm.setText(ctok+ " number was entered! ");
		} else {
			lblConfirm.setText(szLabelConfirmDefault);
		} // if/else

		int ivalue = 0;
		double value = 0.0;
		boolean fError = false;
		// Values from tokens
     	while (st.hasMoreTokens()) {
         	String token = st.nextToken();
         	token = token.replace(',', '.');
         	try {
         		value = Double.parseDouble(token);
			} catch(Exception e) {
				if (fError==false) {
					lblConfirm.setText("'" + token + "'  is not a valid number! ");
					fError = true;
				} // if
				value = Double.NaN;
			} // try/catch
			if (ivalue<cnum) vrvalueNew[ivalue] = value;
         	ivalue++;
     	} // while
     	// Append zeros if not enough tokens
     	while (ivalue<cnum) {
       		vrvalueNew[ivalue] = 0.0;
         	ivalue++;
     	} // while

 	} // parseData


} // class ImportDialog