import javax.swing.*;
import java.awt.*;
import java.awt.event.*;  // MouseListener
import java.util.*;       // Vector

public class LayerPanel extends JPanel implements
					MouseListener,MouseMotionListener {

	// Set up printing nominal scale and pages
	// The printer job will scale and print to paper size
	public static int NUMBER_OF_PAGES_WIDE = 2;
	public static int NUMBER_OF_PAGES_HIGH = 2;
	public static int PAGE_WIDTH = 660;
	public static int PAGE_HEIGHT = (int)(1.414*PAGE_WIDTH);
	public static int  PANEL_WIDTH = NUMBER_OF_PAGES_WIDE*PAGE_WIDTH;
	public static int  PANEL_HEIGHT = NUMBER_OF_PAGES_HIGH*PAGE_HEIGHT;

	protected LayerFrame frameLayer;
	protected ECFTool tool;
	private int ipanel;

	LayerPanel(LayerFrame frameLayer) {
		this.frameLayer = frameLayer;
		ipanel = frameLayer.getIndex();
        setPreferredSize(new Dimension(PANEL_WIDTH,PANEL_HEIGHT));
        setSize(PANEL_WIDTH,PANEL_HEIGHT);
		addMouseListener(this);
		addMouseMotionListener(this);
	} // LayerPanel

    public int getPanelIndex() { return ipanel; }

    public void install(ECFTool tool) {
		if (tool==null) return;
		this.tool = tool;
		setCursor(tool.getMyCursor());
	} // installTool

	// MouseListeners
	public void mouseClicked(MouseEvent e) {;}
	public void mouseEntered(MouseEvent e) {;}
	public void mouseExited(MouseEvent e) {;}

	public void mousePressed(MouseEvent e) {
		if (tool!=null) tool.mousePressed(e,ipanel);
	} // mousePressed
	public void mouseReleased(MouseEvent e) {
		if (tool!=null) tool.mouseReleased(e,ipanel);
	} // mouseReleased

	// MouseMotionListeners
	public void mouseDragged(MouseEvent e) {
		if (tool!=null) tool.mouseDragged(e,ipanel);
	} // mouseDragged
	public void mouseMoved(MouseEvent e) {
		if (tool!=null) tool.mouseMoved(e,this);
	} // mouseMoved

	public String toString() {
		return "LayerPanel #" + ipanel;
	} // toString

	// Paint the contents of this panel
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D)g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		                    RenderingHints.VALUE_ANTIALIAS_ON);
        setForeground(Color.gray);
        g2.fillRect(PANEL_WIDTH, 0, 1000, PANEL_HEIGHT);
        int margin = 5;
        // Draw a rectangular frame
		g2.drawLine(margin,margin,              PANEL_WIDTH-margin,margin);
        g2.drawLine(PANEL_WIDTH-margin,margin,  PANEL_WIDTH-margin,PANEL_HEIGHT-margin);
        g2.drawLine(PANEL_WIDTH-margin,PANEL_HEIGHT-margin, margin,PANEL_HEIGHT-margin);
        g2.drawLine(margin,PANEL_HEIGHT-margin, margin,margin);
        // Draw a horizontal and vertical lines separating pages
        g2.drawLine(margin,PANEL_HEIGHT/2, PANEL_WIDTH-margin,PANEL_HEIGHT/2); // horizintal
        g2.drawLine(PANEL_WIDTH/2,margin, PANEL_WIDTH/2,PANEL_HEIGHT-margin);

//        setForeground(Color.white);
		if (tool!=null) tool.paint(g2,ipanel);
	} // paintComponent

} // LayerPanel