import java.util.*;
/** An iterator for flowconnectors attached to all sides of a node. */
 public class NodeConnectorsIterator implements Iterator {

	Iterator iterEast;
	Iterator iterWest;
	Iterator iterNorth;
	Iterator iterSouth;

	/** Create a node connectors iterator.
	* @param Vector of flowconnectors attached to west side.
	* @param vrconnEast Vector of flowconnectors attached to west side.
	* @param vrconnWest Vector of flowconnectors attached to east side.
	* @param vrconnNorth Vector of flowconnectors attached to north side.
	* @param vrconnSouth Vector of flowconnectors attached to south side.
	*/
	public NodeConnectorsIterator(Vector vrconnEast, Vector vrconnWest,
	                       Vector vrconnNorth, Vector vrconnSouth) {
	iterEast = vrconnEast.iterator();
	iterWest = vrconnWest.iterator();
	iterNorth = vrconnNorth.iterator();
	iterSouth = vrconnSouth.iterator();
	} // NodeConnectorsIterator

	/** Returns true if the iteration has more elements. */
	public boolean hasNext() {
		return iterEast.hasNext() || iterWest.hasNext() || iterNorth.hasNext() || iterSouth.hasNext();
	} // hasNext

	/** Returns the next <a href="FlowConnector.html">flow connector</a> in the interation.
	* @return An object representing a flow connector. Cast as: <p>
	* <code>FlowConnector flowconn = (FlowConnector)iter.next();</code> */
	public Object next() {
		if (iterEast.hasNext()) return iterEast.next();
		if (iterWest.hasNext()) return iterWest.next();
		if (iterNorth.hasNext()) return iterNorth.next();
		if (iterSouth.hasNext()) return iterSouth.next();
		return null;
	} // next

	/** Not used, thorows an UnsupportedOperationException. */
	public void remove() throws UnsupportedOperationException {
		throw new UnsupportedOperationException();}

} // NodeConnectorsIterator