import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** The dialog set the properties of a Price relation.*/
public class PriceDialog extends VariableDialog implements ActionListener {

	/** The price relation to be edited. */
	Price price;
	/** Combo box for money units. */
	JComboBox comboMoneyUnits;
	/** Combo box for commodity units. */
	JComboBox comboCommodityUnits;

	/** Creates a dialog for editing price properties.
	* @param owner The owner of the dialog.
	* @param vrunits A vector of Units.
	* @param price The Price to be edited. */
	public PriceDialog(JDialog owner, Price price) {
		super(owner,"Price properties",price);
//		System.out.println("PriceDialog: 1");
	} // PriceDialog

	/** Set price dialog specific attributes and
	* initialize position and content of the dialog. */
	protected void init() {
		// Get price before proceeding to makeUnitsPanel()
		this.price = (Price)variable;
		// Check box legend.
		szCheckBoxPrescribedLegend = "Prescribe price";
		// Canvas legend.
		szCanvasLegend = "Price as a function of time";
		//
		super.init();
	} // init

	/** Make the units selection panel with combo boxes for
	* money units and commodity units.
	* @return A panel. */
	protected JPanel makeUnitsPanel() {
//		System.out.println("PriceDialog.makeUnitsPanel: 1");
		Iterator iter;

		// Units panel
		JPanel panelUnits= new JPanel(new GridLayout(1,3));

		// Label
//		panelUnits.add(new JLabel("Units:"));

		// Money Units selection combo box
		comboMoneyUnits = new JComboBox();
		Vector vrunitMoney = price.getFlowMoney().getUnits().getUnits();
		iter = vrunitMoney.iterator();
		while (iter.hasNext()) {
			Unit unit = (Unit)iter.next();
			comboMoneyUnits.addItem(unit.getName());
		} // while
		comboMoneyUnits.setSelectedItem(price.getUnitMoney().getName());
		panelUnits.add(comboMoneyUnits);
		comboMoneyUnits.addActionListener(this);

		// Commodity Units selection combo box
		comboCommodityUnits = new JComboBox();
		Vector vrunitCommodity = price.getFlowCommodity().getUnits().getUnits();
		iter = vrunitCommodity.iterator();
		while (iter.hasNext()) {
			Unit unit = (Unit)iter.next();
			comboCommodityUnits.addItem(unit.getName());
		} // while
		comboCommodityUnits.setSelectedItem(price.getUnitCommodity().getName());
		panelUnits.add(comboCommodityUnits);
		comboCommodityUnits.addActionListener(this);

		// Unit label
		labelUnit = new JLabel(" = " + price.getUnits().getNameOfDisplayUnit());
		panelUnits.add(labelUnit);

//		System.out.println("PriceDialog.makeUnitsPanel: end");
		return panelUnits;

	} // makeUnitsPanel

	public void actionPerformed(ActionEvent e) {
		// Set Units of money according to selection in money combo box
		if (e.getSource()==comboMoneyUnits) {
			int iunitMoney = comboMoneyUnits.getSelectedIndex();
			price.setUnitMoney(iunitMoney);
			super.tabelmodel.updateTable();
			repaint();
		} // Money units combobox
		// Set Units of commodity according to selection in commodity combo box
		if (e.getSource()==comboCommodityUnits) {
			int iunitCommodity = comboCommodityUnits.getSelectedIndex();
			price.setUnitCommodity(iunitCommodity);
			super.tabelmodel.updateTable();
			repaint();
		} // Units combobox
		labelUnit.setText(price.getUnits().getNameOfDisplayUnit());

		super.actionPerformed(e);
	} // actionPerformed

} // class PriceDialog