import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
* The dialog defines how the inflows to a process are related to the outflows.
*/
public class ProcessDialog extends JDialog implements ActionListener,FocusListener {

	/** OK button */
	private JButton btnOK;

	/** Number of productivities */
	protected int cproductivity;
	/** Error message on table panel */
	JLabel labelErrMsg;

	/** The process which productivities shall be edited. */
	protected ECFProcess process;
	/** Canvas for flow ratio plot. */
	protected ProcessCanvas processcanvas;
	/** The name of the process. */
	protected String szName;
	/** Text field for editing the name of the process. */
	protected JTextField txtfldName;
	/** Check boxes for prescribed flow ratio. */
	protected JCheckBox[] vrchkboxPrescr;

	/** Vector of inflows */
	protected Vector vrflowIn;
	/** Vector of outflows */
	protected Vector vrflowOut;
	/** Vector of productivities. */
	protected Vector vrproductivity;
	/** Text fields for ratio values. */
	protected JTextField[] vrtxtfldRatio;

	/** Text fields for price names. */
	private Vector vrtxtfldProductivity;
	private Vector vrbtnEdit;
	private Vector vrbtnRemove;

	// Controls
	private Container contentpane;
	private JPanel panelProductivityList;
	private JPanel panelButtons;
	private JComboBox comboboxFlowsIn;
	private JComboBox comboboxFlowsOut;
	private JButton btnAddProductivity;
	private JLabel labelWarnMsg;

	/** Creates a new process dialog.
	* @param owner The parent frame of the dialog.
	* @param process The process which data shall be edited.
	* @param pos The position of the upper left corner of the dialog frame.
	*/
	public ProcessDialog(ECFProcess process, Point pos) {
		super(ECF.frameMain, "ECFProcess properties");
//		System.out.println("*** ProcessDialog: 1.");
		this.process = process;
		szName = process.getName();
		vrproductivity = process.getProductivities();
		setLocation(pos);
		int height = 450 + 50*vrproductivity.size();
		setSize(new Dimension(400,height));
		// Get content pane
		contentpane = getContentPane();
		// Make text fields and buttons for productivities
		int cproductivity = vrproductivity.size();
		vrtxtfldProductivity = new Vector();
		vrbtnEdit = new Vector();
		vrbtnRemove = new Vector();
		try {makeContentPane();}
		catch(Exception e) {System.out.println(e);}
		pack();
//		System.out.println("*** ProcessDialog: end.");
	} // ProcessDialog

	/** Disable last flow coefficient when all other have been set.
	*/
	private void disableLastFlowRatio() {
		// Number of checked flow coeff
		int ccheck = 0;
		for (int ichkbox=0; ichkbox<vrchkboxPrescr.length; ichkbox++) {
			if (vrchkboxPrescr[ichkbox].isSelected()) ccheck++;
			vrchkboxPrescr[ichkbox].setEnabled(true);
		} // for
		// Check if all but one flow ratio was checked
		if (ccheck+1==cproductivity) {
			// Find the last and only flow with no coefficient checked.
			for (int ichkbox=0; ichkbox<vrchkboxPrescr.length; ichkbox++) {
				if (!vrchkboxPrescr[ichkbox].isSelected()) {
					vrchkboxPrescr[ichkbox].setEnabled(false);
				} // if
			} // for
		} // if
	} // disableLastFlowRatio

	/** Adds panels for process name, flows in and out, table with
	* prescribed and calcualted productivities, plots oc productivities and
	* OK and Cancel buttons to the conent pane of the dialog.*/
	protected void makeContentPane() {
		contentpane.setLayout(new BoxLayout(contentpane, BoxLayout.Y_AXIS ));
		// Name of process
		contentpane.add(makeNamePanel());
		// Flows
		contentpane.add(makeAddProductivityPanel());
		// Productivities
		panelProductivityList = makeProductivityListPanel();
		contentpane.add(panelProductivityList);
		// OK and Cancel buttons
		panelButtons = makeButtonsPanel();
		contentpane.add(panelButtons);
	} // makeContentPane

	/** Makes a process name panel.
	* @return A panel with a text field for editing the process name. */
	private JPanel makeNamePanel() {
		JPanel panelName = new JPanel(new BorderLayout());
		// Label
		JLabel label = new JLabel("Process Name:");
		panelName.add(label,BorderLayout.NORTH);
		// Text field
		txtfldName = new JTextField(szName,20);
		panelName.add(txtfldName, BorderLayout.CENTER);
		return panelName;
	} // makeNamePanel

	/** Makes a panel for listing flows in and flows out.
	* @return A panel ???.
	*/
	private JPanel makeAddProductivityPanel() {

		JPanel panelFlows = new JPanel(new BorderLayout());
		panelFlows.setBorder(BorderFactory.createTitledBorder(
                                    "Add a productivity relation:"));
		Font fontHeadings = new Font("Arial", Font.BOLD, 12);
		Iterator iter;

		// List flows in and out
		vrflowIn = new Vector();
		vrflowOut = new Vector();
		NodeConnectors conns = process.getNodeConnectors();
		iter = conns.getNodeConnectorsIterator();
		// Find flows in or out
		while (iter.hasNext()) {
			FlowConnector conn = (FlowConnector)iter.next();
			FlowPath flowpath = conn.getFlowPath();
			if (conn.isHeadEnd())
				vrflowIn.add(flowpath.getFlow());
			if (conn.isTailEnd())
				vrflowOut.add(flowpath.getFlow());
		} // while

		// Comboboxes for money and commodity layer selection
		// --------------------------------------------------
		// Selection panel
		JPanel panelSelection = new JPanel(new GridLayout(2,2));
		// Headings
		panelSelection.add(new JLabel("Flows in"));
		panelSelection.add(new JLabel("Flows out"));

		Vector vrszFlowIn = new Vector();
		iter = vrflowIn.iterator();
		while (iter.hasNext()) vrszFlowIn.add(((Flow)iter.next()).getName());
		comboboxFlowsIn = new JComboBox(vrszFlowIn);
		panelSelection.add(comboboxFlowsIn);

		Vector vrszFlowOut = new Vector();
		iter = vrflowOut.iterator();
		while (iter.hasNext()) vrszFlowOut.add(((Flow)iter.next()).getName());
		comboboxFlowsOut = new JComboBox(vrszFlowOut);
		panelSelection.add(comboboxFlowsOut);

		panelFlows.add(panelSelection,BorderLayout.NORTH);

		// Add warning message label
		labelWarnMsg = new JLabel(" ");
		labelWarnMsg.setForeground(Color.red);
		labelWarnMsg.setVisible(true);
		panelFlows.add(labelWarnMsg,BorderLayout.CENTER);

		// Add add productivity button
		btnAddProductivity = new JButton("Add productivity");
		btnAddProductivity.setEnabled(true);  // Add productivity button enabled as default
		disableAddProductivityButtonIfFlowsUsed();
		panelFlows.add(btnAddProductivity,BorderLayout.SOUTH);

		// Add action listeners
		//---------------------
		btnAddProductivity.addActionListener(this);
        return panelFlows;
	} // makeAddProductivityPanel

	/** Makes a panel for listing productivities.
	* @return A panel ???.
	*/
	private JPanel makeProductivityListPanel() {
		// List of productivities
		JLabel label;
		JButton btn;
		Font fontHeadings = new Font("Arial", Font.BOLD, 12);
		int cproductivity = vrproductivity.size();
		JPanel panelListOfProductivities = new JPanel(new GridLayout(1+cproductivity,5,10,0));

		// Headings
		label = new JLabel("Flows in");
		label.setFont(fontHeadings);
		panelListOfProductivities.add(label);
		label = new JLabel("Flows Out");
		label.setFont(fontHeadings);
		panelListOfProductivities.add(label);
		label = new JLabel("Name of productivity");
		label.setFont(fontHeadings);
		panelListOfProductivities.add(label);
		// Fill remaining cells of heading line
		panelListOfProductivities.add(new JLabel(""));
		panelListOfProductivities.add(new JLabel(""));

		// Productivities
		Iterator iter = vrproductivity.iterator();
		int iproductivity=0;
		vrtxtfldProductivity.clear();
		vrbtnEdit.clear();
		vrbtnRemove.clear();
		while (iter.hasNext()) {
			Productivity productivity = (Productivity)iter.next();
			// Names of money and commodity flows
			panelListOfProductivities.add(new JLabel(productivity.getFlowDenominator().getName()));
			panelListOfProductivities.add(new JLabel(productivity.getFlowNominator().getName()));
			// Name text field
			JTextField txtfld = new JTextField(productivity.getName());
			txtfld.setEnabled(true);
			vrtxtfldProductivity.add(txtfld);
			panelListOfProductivities.add(txtfld);
			// Edit productivity button
			btn = new JButton("Edit...");
			btn.addActionListener(this);
			vrbtnEdit.add(btn);
			panelListOfProductivities.add(btn);
			// Remove productivity button
			btn = new JButton("Remove");
			btn.addActionListener(this);
			vrbtnRemove.add(btn);
			panelListOfProductivities.add(btn);
			// Add focus listener
			txtfld.addFocusListener(this);
			// Add actionlisteners
			txtfld.addActionListener(this);
			iproductivity++;
		} // while
		// Create border
        panelListOfProductivities.setBorder(BorderFactory.createTitledBorder(
                                    "Edit productivities:"));
		return panelListOfProductivities;
	} // makeProductivityListPanel

	private JPanel makeButtonsPanel() {

		panelButtons = new JPanel(new FlowLayout());
		// Buttons
		btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		panelButtons.add(btnOK);
		return panelButtons;
	} // makeButtonsPanel

	private boolean checkIfFlowMayBeAdded(Flow flowIn, Flow flowOut) {
		boolean fOK=true;
		Iterator iter = vrproductivity.iterator();
		while (iter.hasNext()) {
			Productivity productivity = (Productivity)iter.next();
//			System.out.println("ProcessDialog.checkIfFlowMayBeAdded: " +
//				productivity);
			if (flowIn==productivity.getFlowDenominator() &&
				flowOut==productivity.getFlowNominator())
				fOK=false;
		} // while
//		System.out.println("ProcessDialog.checkIfFlowMayBeAdded: fOK=" + fOK);
		return fOK;
	} // checkIfFlowMayBeAdded

	/** Disable "Add price" button if either money flow or
	* commodity flow alredy is used in a price relation. */
	private void disableAddProductivityButtonIfFlowsUsed() {
		if (vrflowIn.size()==0 || vrflowOut.size()==0 ||
			vrproductivity.size()+1==vrflowIn.size()+vrflowOut.size())
		btnAddProductivity.setEnabled(false);
	} // disableAddProductivityButtonIfFlowsUsed

	public void showErrorMsg(String errmsg) {
		labelErrMsg.setText(errmsg);
	} // showErrorMsg

	/** Refrech list of productivities. */
	private void refreshListOfProductivities() {
		// Remove the two last panels
		contentpane.remove(panelProductivityList);
		contentpane.remove(panelButtons);
		// Remake and add the two last panels
		panelProductivityList = makeProductivityListPanel();
		contentpane.add(panelProductivityList);
		contentpane.add(panelButtons);
		pack();
	} // refreshListOfProductivities

	/** Open a dialog for setting properties of a productivity.
	* @param productivity The Price which properties shall be edited. */
	public void openProductivityDialog(Productivity productivity) {
		ProductivityDialog dialog = new ProductivityDialog(this,productivity);
	} // openProductivityDialog

	/** Invoked when a txtfldProductivity gains the keyboard focus,
	* does nothing*/
	public void focusGained(FocusEvent e) {
	} // focusGained

	/** Invoked when a text field for setting a productivity name
	* gains the keyboard focus, sets the name of a <a href="Productivity.html">productivity</a>.
	* @param e The event that activated the FocusListener.
	*/
	public void focusLost(FocusEvent e) {
		// Find the invoking text field and set productivity name.
		int iproductivity = 0;
		Iterator iter = vrproductivity.iterator();
		while (iter.hasNext()) {
			Productivity productivity = (Productivity)iter.next();
			JTextField txtfld = (JTextField)vrtxtfldProductivity.elementAt(iproductivity);
			if (e.getSource()==txtfld)    {
				productivity.setName(((JTextField)txtfld).getText());
			} // if
			iproductivity++;
		} // while
	} // focusLost

	/** Respond to checkboxes and buttons. */
	public void actionPerformed(ActionEvent e) {
		int iproductivity;

		// No warning text at start
		if (labelWarnMsg!=null) labelWarnMsg.setText(" ");
		// Add productivity button enabled as default
		if (btnAddProductivity!=null) btnAddProductivity.setEnabled(true);

		// Add a new productivity relation
		if (e.getSource()==btnAddProductivity) {
			Productivity productivity = new Productivity(process);
			try {
				Flow flowIn = (Flow)vrflowIn.elementAt(comboboxFlowsIn.getSelectedIndex());
				Flow flowOut = (Flow)vrflowOut.elementAt(comboboxFlowsOut.getSelectedIndex());
//				System.out.println("ProcessDialog.actionPerformed: 1");
//				System.out.println("ProcessDialog.actionPerformed: 2 fOK=" +
//					checkIfFlowMayBeAdded(flowIn,flowOut));
				// Check if flowIn and flowOut may be used
				if (checkIfFlowMayBeAdded(flowIn,flowOut)) {
//					System.out.println("ProcessDialog.actionPerformed: 3");
					productivity.setFlowIn(flowIn);
					productivity.setFlowOut(flowOut);
					vrproductivity.add(productivity);
					refreshListOfProductivities();
					disableAddProductivityButtonIfFlowsUsed();
//					System.out.println("ProcessDialog.actionPerformed: 4");
				} else {
					String szMsg = flowOut.getName() + " and " + flowIn.getName()
						+ " have been used before in a productivity relation.";
					// JOptionPane.showMessageDialog(this,szMsg);
					if (labelWarnMsg!=null) labelWarnMsg.setText(szMsg);
				} // if/else
			} catch(NoUnitException excNoUnit) {
				if (labelWarnMsg!=null) labelWarnMsg.setText("Flow '" +
					excNoUnit.getMessage() +
					"' has no units. Assign units before creating a productivity.");
			} catch(Exception excAdd) {
				// The catch clause will find a variety of null pointer exceptions
				// and index out of bounds exceptions.
				// Instead of checking beforehand, it is easier with try and catch.
				System.out.println("ProcessDialog.actionPerformed: \n" + excAdd);
			} // try/catch
		} // if

		// Edit productivity and Remove productivity buttons
		//} else {
			JButton btn;
			iproductivity = 0;
			Iterator iter = vrproductivity.iterator();
			while (iter.hasNext()) {
				Productivity productivity = (Productivity)iter.next();
				btn = (JButton)vrbtnEdit.elementAt(iproductivity);
				if (e.getSource()==btn)    {
					openProductivityDialog(productivity);
					break;
				} // if
				btn = (JButton)vrbtnRemove.elementAt(iproductivity);
				if (e.getSource()==btn)    {
					vrproductivity.remove(productivity);
					Calculator.removeProductivity(productivity);
					break;
				} // if
				iproductivity++;
			} // while
			try {
				refreshListOfProductivities();
				// Check if flows are used in productivitys
				disableAddProductivityButtonIfFlowsUsed();
			} catch(Exception excAdd) {
				System.out.println("ProcessDialog.actionPerformed: edit/remove productivity\n" + excAdd);
			} // try/catch

		//} // if

    	if (e.getSource()==btnOK) {
			// Set name of process
			process.setName(txtfldName.getText());
			// Ratio names, Check boxes
			for (iproductivity=0; iproductivity<cproductivity; iproductivity++) {
				FlowRatio productivity = (FlowRatio)vrproductivity.elementAt(iproductivity);
				productivity.setName(vrtxtfldRatio[iproductivity].getText());
			} // for
			dispose();
			Calculator.calculate();
		} // if btnOK

	} // actionPerformed

} // class ProcessDialog