import java.util.*;         // Vector
import javax.swing.table.*; // DefaultTableModel
import javax.swing.event.*; // TableModelEvent
import java.text.ParseException;

/**
* The ProcessTableModel stores the values
* of the <A href="FlowRatio.html">FlowRatios</A> to be displayed by a
* ProcessDialog.
*/

public class ProcessTableModel extends DefaultTableModel implements TableModelListener {

	/** The dialog in which the data is dispalyed. */
	protected ProcessDialog dialog;
	/** A vector of <A href="FlowRatio.html">FlowRatios</A>. */
	protected Vector vrflowratio;
	/** A vector of column names (headings). */
	protected Vector vrnameColumn = new Vector();
	/** A vector of vectors of row data*/
	protected Vector vrData = new Vector();
	/** An array of vectors of calculated row data.*/
	protected Vector[] vrvrDataCalculated;
	/** A vector of prescribed row data.*/
	protected Vector[] vrvrDataPrescribed;
	/** An array of calculated display values.*/
	protected double vrvalueCalculated[];
	/** Vector of time points.*/
	private Vector vrtimeInc;
	//** Number of time increments.*/
	private int ctime = 0;

	/**
	* Creates a ProcessTableModel that that is used by a ProcessDialog to
	* display an edit the values of FlowRatios.
	*/
    public ProcessTableModel(ProcessDialog dialog, Vector vrflowratio) throws Exception {

		this.dialog = dialog;
		this.vrflowratio = vrflowratio;
		int cratio = vrflowratio.size();
		if (cratio==0) throw new Exception("Splitter Table Model: No flow ratios found.");
		int iratio;
		int itime;
		System.out.println("*** ProcessTableModel: to be converted from splitter to process.");

		// Time axis
		// Set row labels
		vrnameColumn.add("Time");
		vrnameColumn.add("year");
		// Get time values
		vrtimeInc = ((FlowRatio)vrflowratio.firstElement()).getTimeAxis().getTimeIncrements();
		ctime = vrtimeInc.size()-1;
		for (itime=0; itime<ctime; itime++) {
			TimeIncrement timeinc = (TimeIncrement)vrtimeInc.elementAt(itime);
			vrnameColumn.add(String.valueOf(timeinc.getStartTime()));
		} // for

		// Flow ratio data
		vrvrDataPrescribed = new Vector[vrflowratio.size()];
		vrvrDataCalculated = new Vector[vrflowratio.size()];
		Iterator iter = vrflowratio.iterator();
		iratio = 0;
		while (iter.hasNext()) {

			// One flow ratio as a function of time
			FlowRatio flowratio = (FlowRatio)iter.next();
			// Fetch values
			String szName = flowratio.getName();
			double[][] mxvalueDisplay = flowratio.getDisplayValues();

			try {
			// Prescribed values
			vrvrDataPrescribed[iratio] = new Vector();
			vrvrDataPrescribed[iratio].add(szName);
			String szPrescr = "";
			if (flowratio.isPrescribed()) szPrescr = "Prescribed";
			vrvrDataPrescribed[iratio].add(szPrescr);
			for (itime=0; itime<ctime; itime++) {
				vrvrDataPrescribed[iratio].add(Quantity.formatDouble(mxvalueDisplay[0][itime]));
			} // for
			vrData.add(vrvrDataPrescribed[iratio]);
//			System.out.println("ProcessTableModel: vrvrDataPrescribed done\n");

			// Calculated values
			vrvrDataCalculated[iratio] = new Vector();
			vrvrDataCalculated[iratio].add(szName);
			vrvrDataCalculated[iratio].add("Calculated");
			for (itime=0; itime<ctime; itime++) {
				vrvrDataCalculated[iratio].add(Quantity.formatDouble(mxvalueDisplay[1][itime]));
			} // for
			vrData.add(vrvrDataCalculated[iratio]);
//			System.out.println("ProcessTableModel: vrvrDataCalculated done\n");
			} catch(Exception e) {
				System.out.println("ProcessTableModel: " + e);
			} // try/catch
			iratio++;

		} // while

		// Set up tabel data
		setDataVector(vrData,vrnameColumn);
		// Add listener
		addTableModelListener(this);

	} // ProcessTableModel

    /**
    * JTable uses this method to determine the default renderer
    * editor for each cell.
    * @param icol index of table column.
    * @return The Class of the column data.
    */
    public Class getColumnClass(int icol) {
		try {
        	return getValueAt(0, icol).getClass();
		} catch(Exception e) {
			System.out.println(e);
			return null;
		} // try
    } // getColumnClass

    /* Don't need to implement this method unless your table's editable. */
    /**
    * Tell whether a cell of the table is editable.
    * @param irow index of table row (except headings).
    * @param icol index of table column.
    * @return true if cell i editable, else false.
    */
    public boolean isCellEditable(int irow, int icol) {
        //Note that the data/cell address is constant,
        //no matter where the cell appears onscreen.
        if (icol<=1) return false;  // Two first columns are row labels
        else {
			int iratio = (int)irow/2; // One ratio per two rows
			FlowRatio flowratio = (FlowRatio)vrflowratio.elementAt(iratio);
			// Only even (0,2,...) rows represent prescribed values and
			// can be editable
			if ((irow==2*iratio) && flowratio.isPrescribed()) return true;
        	else return false;
		} // if icol
    } // isCellEditable

	/** Update the flow ratios when the table is changed.
	* Repaints the dialog if successfull.
	* @param e table model event.
	*/ // Method of interface TableModelListener
	public void tableChanged(TableModelEvent e) {

		/** An array of prescribed display values.*/
		double vrvaluePrescribed[] = new double[ctime];

		// Check all FlowRatios
		Iterator iter = vrflowratio.iterator();
		int irow = 0;
		int iratio = 0;
		boolean fOK = true;
		while (iter.hasNext() && fOK) {
			FlowRatio flowratio = (FlowRatio)iter.next();
			String szName = flowratio.getName();
			String szPrescr = "";
			if (flowratio.isPrescribed()) szPrescr = "Prescribed";
			// Prescribed values
			vrvrDataPrescribed[iratio].set(0,szName);
			vrvrDataPrescribed[iratio].set(1,szPrescr);
			// Calculated values
			vrvrDataCalculated[iratio].set(0,szName);
			// FlowRatios
			String szCell = "0.0";
			for (int itime=0; itime<ctime; itime++) {
				TimeIncrement timeinc = (TimeIncrement)vrtimeInc.elementAt(itime);
				String szTime = String.valueOf(timeinc.getStartTime());
				try {
					szCell = (String)getValueAt(irow,2+itime);
//					System.out.print(szCell + " ");
					vrvaluePrescribed[itime] = Quantity.parseDouble(szCell);
					dialog.showErrorMsg(" ");
				} catch(ParseException pex) {
					dialog.showErrorMsg("\"" + szCell + "\" at time \"" +
						szTime + "\" is not a valid decimal number.");
					fOK = false;
					break;
				} catch(Exception ex) {
					dialog.showErrorMsg("Unexpected error while parsing \"" + szCell +
					"\" at time \"" + szTime + "\": " + ex);
					fOK = false;
					break;
				} // try/catch
			} // for itime
			irow += 2;
			iratio++;
			if (fOK) flowratio.setDisplayValuesPrescribed(vrvaluePrescribed);
		} // while vrflowratio
		dialog.repaint();
	} // tableChanged

} // ProcessTableModel