import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** The dialog sets the properties of a Productivity relation.
* Productivity is the ratio between outputs (products) and
* inputs (work or other products) to a process.*/
public class ProductivityDialog extends VariableDialog implements ActionListener {

	/** The productivity relation to be edited. */
	Productivity productivity;
	/** Combo box for product units. */
	JComboBox comboProductUnits;
	/** Combo box for input units. */
	JComboBox comboInputUnits;

	/** Creates a dialog for editing productivity properties.
	* @param owner The owner of the dialog.
	* @param productivity The Productivity to be edited. */
	public ProductivityDialog(JDialog owner, Productivity productivity) {
		super(owner,"Productivity properties",productivity);
		//this.productivity = productivity;
	} // ProductivityDialog

	/** Set productivity dialog specific attributes and
	* initialize position and content of the dialog. */
	protected void init() {
		// Get productivity before proceeding to makeUnitsPanel()
		this.productivity = (Productivity)variable;
		// Check box legend.
		szCheckBoxPrescribedLegend = "Prescribe productivity";
		// Canvas legend.
		szCanvasLegend = "Productivity as a function of time";
		//
		super.init();
	} // init

	/** Make the units selection panel with combo boxes for
	* products units and input units.
	* @return A panel. */
	protected JPanel makeUnitsPanel() {
		Iterator iter;
		Unit unit;

		// Units panel
		JPanel panelUnits= new JPanel(new GridLayout(1,3));

		// Product Units selection combo box
		comboProductUnits = new JComboBox();
		Vector vrunitProduct = productivity.getFlowNominator().getUnits().getUnits();
		iter = vrunitProduct.iterator();
		while (iter.hasNext()) {
			unit = (Unit)iter.next();
			comboProductUnits.addItem(unit.getName());
		} // while

		comboProductUnits.setSelectedItem(productivity.getUnitNominator().getName());
		panelUnits.add(comboProductUnits);
		comboProductUnits.addActionListener(this);

		// Commodity Units selection combo box
		comboInputUnits = new JComboBox();
		Vector vrunitInput = productivity.getFlowDenominator().getUnits().getUnits();
		iter = vrunitInput.iterator();
		while (iter.hasNext()) {
			unit = (Unit)iter.next();
			comboInputUnits.addItem(unit.getName());
		} // while
		comboInputUnits.setSelectedItem(productivity.getUnitDenominator().getName());
		panelUnits.add(comboInputUnits);
		comboInputUnits.addActionListener(this);

		// Unit label
		labelUnit = new JLabel(" = " + productivity.getUnits().getNameOfDisplayUnit());
		panelUnits.add(labelUnit);

		return panelUnits;

	} // makeUnitsPanel

	public void actionPerformed(ActionEvent e) {
		// Set Units of products according to selection in products combo box
		if (e.getSource()==comboProductUnits) {
			int iunitNominator = comboProductUnits.getSelectedIndex();
			productivity.setUnitNominator(iunitNominator);
			super.tabelmodel.updateTable();
			repaint();
		} // Product units combobox
		// Set Units of input according to selection in input combo box
		if (e.getSource()==comboInputUnits) {
			int iunitDenominator = comboInputUnits.getSelectedIndex();
			productivity.setUnitDenominator(iunitDenominator);
			super.tabelmodel.updateTable();
			repaint();
		} // Units combobox
		labelUnit.setText(productivity.getUnits().getNameOfDisplayUnit());

		super.actionPerformed(e);
	} // actionPerformed

} // class ProductivityDialog