import java.awt.*;        // Container, BorderLayout
import java.awt.event.*;  // WindowAdapter
import javax.swing.*;     // JFrame, JPanel
import java.util.*;       // Vector

public class QuantityFrame extends JFrame implements ActionListener {

	private Quantity quant;
	private Units units;
	private Vector vrquant;
	private Container panelContent;
	private JComboBox combo;
	private JTextField txtfdMax;
	private int cquant;
	private JTextField vrtxtfd[];
	private ValueBar vrbar[];

	public QuantityFrame(String szTitle, Units units, Vector vrquant) {
		super(szTitle);
		this.units = units;
		this.vrquant = vrquant;
		cquant = vrquant.size();
		vrtxtfd = new JTextField[cquant];
		vrbar = new ValueBar[cquant];
		panelContent = getContentPane();
		GridBagLayout layoutGB = new GridBagLayout();
		JPanel panelGB = new JPanel(layoutGB);
		panelContent.add(panelGB);
		GridBagConstraints constrGB;

		// Units
		constrGB = new GridBagConstraints();
		// Label
		JLabel label = new JLabel(units.getName());
		constrGB.ipadx = 10;
		layoutGB.setConstraints(label,constrGB);
		panelGB.add(label);

		// Combobox
		Vector vrComboItem = new Vector();
		Iterator iter = units.iterator();
		while (iter.hasNext()) {
			Unit unit = (Unit)iter.next();
			vrComboItem.add(unit.getName());
		} // while
		combo = new JComboBox(vrComboItem);
		combo.addActionListener(this) ;
		constrGB.gridx = 1;
		constrGB.ipadx = 10;
		constrGB.anchor = GridBagConstraints.WEST;
		layoutGB.setConstraints(combo,constrGB);
		panelGB.add(combo);

		// Max scale
		JPanel panelMax = new JPanel();
		JLabel labelMax = new JLabel("Max: ");
		txtfdMax = new JTextField(10);
		txtfdMax.setText(units.getStringMaxValue());
		txtfdMax.addActionListener(this);
		panelMax.add(labelMax);
		panelMax.add(txtfdMax);
		constrGB.gridx = 2;
		constrGB.ipadx = 0;
		layoutGB.setConstraints(panelMax,constrGB);
		panelGB.add(panelMax);

		// Quantities
		constrGB = new GridBagConstraints();
		constrGB.gridy = 1;
		for (int iquant=0; iquant<cquant; iquant++) {

			// Label
			quant= (Quantity)vrquant.elementAt(iquant);
			label = new JLabel(quant.getName());
			constrGB.gridx = 0;
			constrGB.ipadx = 10;
			constrGB.anchor = GridBagConstraints.WEST;
			layoutGB.setConstraints(label,constrGB);
			panelGB.add(label);

			// Value in text field
			vrtxtfd[iquant] = new JTextField(10);
			vrtxtfd[iquant].setText(quant.getStringQuant());
			constrGB.gridx = 1;
			constrGB.ipadx = 10;
			layoutGB.setConstraints(vrtxtfd[iquant],constrGB);
			panelGB.add(vrtxtfd[iquant]);

			// Bar display
			vrbar[iquant] = new ValueBar();
			vrbar[iquant].setPixels(quant.getPixelValue(100));
			constrGB.gridx = 2;
			layoutGB.setConstraints(vrbar[iquant],constrGB);
			panelGB.add(vrbar[iquant]);

			constrGB.gridy++;
		} // for
		combo.setSelectedIndex(units.getIndexOfDisplayUnit());

		pack();
		show();
		// Close frame
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
				System.exit(0);
			} // windowClosing
		}); // addWindowListener
	} // QuantityFrame

	public void actionPerformed(ActionEvent e) {
		// Combo box event
		if (e.getSource()==combo) {
			units.setDisplayUnit((String)combo.getSelectedItem());
			for (int iquant=0; iquant<cquant; iquant++) {
				quant = (Quantity)vrquant.elementAt(iquant);
				vrtxtfd[iquant].setText(quant.getStringQuant());
			} // for
			txtfdMax.setText(units.getStringMaxValue());
		}  // Combo box event

		// Max text field event
		if (e.getSource()==txtfdMax) {
			double valueMax = Double.parseDouble(txtfdMax.getText());
			units.setScaleToPixelsMax(valueMax);
			for (int iquant=0; iquant<cquant; iquant++) {
				quant= (Quantity)vrquant.elementAt(iquant);
				vrbar[iquant].setPixels(quant.getPixelValue(100));
				vrbar[iquant].repaint();
			} // for
		} // Max text field event
	} // actionPerformed

	private class ValueBar extends Canvas {

		private int pixels = 50;

		protected ValueBar() {
			setSize(100,10);
			setBackground(Color.white);
		} // ValueBar

		public void setPixels(int pixels) { this.pixels = pixels; }

		public void paint(Graphics g) {
			g.setColor(Color.red);
			g.fillRect(0,0,pixels,10);
		} // paint

	} // ValueBar

} // QuantityFrame

