import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.swing.*;      // JFileChooser
import javax.imageio.ImageIO;
import javax.imageio.IIOException;

public class RenderedImage {

	JFileChooser chooserJPG = new JFileChooser();

	RenderedImage() {
		System.out.println("RenderedImage: 1");
		chooserJPG.setFileFilter(new ECFFileFilter("jpg"));
		chooserJPG.setCurrentDirectory(new File("."));
		int clayer = ECF.getNoOfLayers();

		for (int ilayer=0; ilayer<clayer; ilayer++) {
			renderImage(ilayer);
		} // for

		System.out.println("RenderedImage: end");
	} // RenderedImage

	protected void renderImage(int ilayer) {
		System.out.println("RenderedImage.renderImage: 1");

		// File chooserJPG
		String szFullName = ECFMainFrame.getFileName();
		String szModelLayerName = ECF.modelOpen.getName() + "_" + ECF.getLayer(ilayer).getName();
		szModelLayerName = szModelLayerName.replace(' ','_');
		String szNameFile = ECFUtil.getFileNameWithoutExtension(szFullName) +
			"_" + szModelLayerName + ".jpg";
		System.out.println("RenderedImage.renderImage: szNameFile = " + szNameFile);
		chooserJPG.setSelectedFile(new File(szNameFile));
		String szPath = System.getProperty("user.dir");
		System.out.println("RenderedImage.renderImage: szPath = " + szPath);
		int returnVal = chooserJPG.showSaveDialog(ECF.frameMain);
		if(returnVal == JFileChooser.APPROVE_OPTION) {

    		// Write generated image to a file
    		try {
				// Create an image to save
		    	BufferedImage rendImage = createImage(ilayer);
	  		    // Save as JPEG
				szPath = chooserJPG.getSelectedFile().getAbsolutePath();
				System.out.println("RenderedImage.renderImage: to jpeg file = " + szPath);
    		    File file = new File(szPath);
    		    ImageIO.write(rendImage, "jpg", file);
    		    rendImage.flush();
    		    rendImage = null;
				System.out.println("RenderedImage.renderImage: jpg file done");

    		} catch (IOException e) {
				System.out.println("RenderedImage.renderImage:" + e);
				String szMsg = "Could not save image to " + szPath + " '\n" + e.getMessage();
				JOptionPane.showMessageDialog(ECF.frameMain,szMsg);
    		} // try/catch
		} // if returnVal

		System.out.println("renderImage: end");
	} // renderImage

    /** Returns a generated image for one layer.
    * @param ilayer Index of ther layer to be printed.
    * @return A buffered image.
    */
    public BufferedImage createImage(int ilayer) {

		double factorEnhancement = 1.5;
        int width = (int)factorEnhancement*LayerPanel.PANEL_WIDTH;
        int height = (int)factorEnhancement*LayerPanel.PANEL_HEIGHT;

        // Create a buffered image in which to draw
        BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // Create a graphics contents on the buffered image
        Graphics2D g2d = bufferedImage.createGraphics();

        // Draw graphics
		double sc = factorEnhancement*ECFModel.scaleDisplay;
		System.out.println("RenderedImage.createImage: sc=" + sc);
        g2d.setColor(Color.white);
        g2d.fillRect(0, 0, width, height);
		g2d.scale(sc,sc);
        ECF.modelOpen.paint(g2d,ilayer);

        // Graphics context no longer needed so dispose it
        g2d.dispose();

        return bufferedImage;
    }

	public static void main(String[] args) {
		System.out.println("Welcome to TryImage");
		RenderedImage image = new RenderedImage();
	} // main

} // class RenderedImage