import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
* The dialog defines how the sum of inflows to a splitter is divided
* between the outflows.
*/
public class SplitterDialog extends JDialog implements ActionListener,FocusListener {
//public class SplitterDialog extends VariableDialog implements ActionListener,FocusListener {

	/** OK button */
	private JButton btnOK;

	/** Number of flow ratios */
	protected int cflowratio;
	/** Error message on table panel */
	JLabel labelErrMsg;

	/** The splitter which flow ratios shall be edited. */
	protected Splitter splitter;
	/** Canvas for flow ratio plot. */
	private   SplitterCanvas splittercanvas;
	/** The name of the splitter*/
	protected String szName;
	/** Text field for editing the name of the splitter. */
	protected JTextField txtfldName;

	/** Vector of inflows */
	protected Vector vrflowIn;
	/** Vector of flow ratios. */
	protected Vector vrflowratio;
	/** Text fields for ratio values. */
	protected JTextField[] vrtxtfldRatio;
	/** Check boxes for prescribed flow ratio. */
	protected JCheckBox[] vrchkboxPrescr;
	/** Buttons for data import. */
	protected JButton[] vrbtnImport;
	/** Table data model. */
	private SplitterTableModel tabelmodel = null;

	/** Creates a new splitter dialog.
	* @param owner The parent frame of the dialog.
	* @param splitter The splitter which data shall be edited.
	* @param pos The position of the upper left corner of the dialog frame.
	*/
	public SplitterDialog(Splitter splitter, Point pos) {
		super(ECF.frameMain, "Splitter properties", true);
		this.splitter = splitter;
		szName = splitter.getName();
		vrflowratio = splitter.getFlowRatios();
		setLocation(pos);
		int height = 450 + 50*vrflowratio.size();
		setSize(new Dimension(500,height));
		try {makeContentPane();}
		catch(Exception e) {System.out.println(e);}
		pack();
		ECFUtil.center(this);
	} // SplitterDialog

	/** Disable last flow coefficient when all other have been set.
	*/
	private void disableLastFlowRatio() {
		// Number of checked flow coeff
		int ccheck = 0;
		for (int ichkbox=0; ichkbox<vrchkboxPrescr.length; ichkbox++) {
			if (vrchkboxPrescr[ichkbox].isSelected()) ccheck++;
			vrchkboxPrescr[ichkbox].setEnabled(true);
		} // for
		// Check if all but one flow ratio was checked
		if (ccheck+1==cflowratio) {
			// Find the last and only flow with no coefficient checked.
			for (int ichkbox=0; ichkbox<vrchkboxPrescr.length; ichkbox++) {
				if (!vrchkboxPrescr[ichkbox].isSelected()) {
					vrchkboxPrescr[ichkbox].setEnabled(false);
				} // if
			} // for
		} // if
	} // disableLastFlowRatio

	/** Adds panels for splitter name, flows in and out, table with
	* prescribed and calcualted flow ratios, plots oc flow ratios and
	* OK and Cancel buttons to the conent pane of the dialog.*/
	protected void makeContentPane() {
		Container pane = getContentPane();
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS ));
		// Name of splitter
		pane.add(makeNamePanel());
		// Flows
		pane.add(makeFlowsPanel());
		// Add table panel
		pane.add(makeTablePanel());
		// Add plot panel
		pane.add(makePlotPanel());
		// OK and Cancel buttons
		pane.add(makeButtonsPanel());
	} // makeContentPane

	/** Makes a splitter name panel.
	* @return A panel with a text field for editing the splitter name. */
	private JPanel makeNamePanel() {
		JPanel panelName = new JPanel(new BorderLayout());
		// Label
		JLabel label = new JLabel("Splitter Name:");
		panelName.add(label,BorderLayout.NORTH);
		// Text field
		txtfldName = new JTextField(szName,20);
		panelName.add(txtfldName, BorderLayout.CENTER);
		return panelName;
	} // makeNamePanel

	/** Makes a panel for listing flow in and flows out.
	* Prescribed flow ratios can be checked.
	* @return A panel with a text fields for flow ratio names and
	*         check boxes for prescribed flow ratios.
	*/
	private JPanel makeFlowsPanel() {

		JLabel label;
		Font fontHeadings = new Font("Arial", Font.BOLD, 12);
		Iterator iter;

		// List flows in
		vrflowIn = new Vector();
		NodeConnectors conns = splitter.getNodeConnectors();
		iter = conns.getNodeConnectorsIterator();
		// Find flows in
		while (iter.hasNext()) {
			FlowConnector conn = (FlowConnector)iter.next();
			FlowPath flowpath = conn.getFlowPath();
			if (conn.isHeadEnd())
				vrflowIn.add(flowpath.getFlow());
		} // while

		// Make text fields and check boxes for flow out (flow ratios).
		cflowratio = vrflowratio.size();
		vrtxtfldRatio = new JTextField[cflowratio];
		vrchkboxPrescr = new JCheckBox[cflowratio];
		vrbtnImport = new JButton[cflowratio];

		// Make a panel with three columns and as many rows as needed
		JPanel panelFlows = new JPanel();
		GridBagLayout gblayout = new GridBagLayout();
		panelFlows.setLayout(gblayout);
		panelFlows.setBorder(BorderFactory.createTitledBorder(
                                    "Set flow ratios"));
        GridBagConstraints con;

        // Flows in
        JPanel panelFlowsIn = new JPanel();
		panelFlowsIn.setLayout(new BoxLayout(panelFlowsIn,BoxLayout.Y_AXIS));
		// Heading
		label = new JLabel("Flows in   ");
		label.setFont(fontHeadings);
		panelFlowsIn.add(label);
		// Names of inflows
		iter = vrflowIn.iterator();
		while (iter.hasNext()) {
			Flow flow = (Flow)iter.next();
			panelFlowsIn.add(new JLabel(flow.getName()));
		} // while
		con = new GridBagConstraints();
		con.gridx = 0;
		con.gridheight = GridBagConstraints.REMAINDER;
		gblayout.setConstraints(panelFlowsIn,con);
		panelFlows.add(panelFlowsIn);

		// Splitter symbol
		ImageIcon icon = new ImageIcon(SplitterDialog.class.getResource("SplitterBtn.gif"));
		label = new JLabel("  ",icon,SwingConstants.CENTER);
		con.gridx = 1;
		gblayout.setConstraints(label,con);
		panelFlows.add(label);

		// Flows out
		// Headings
		label = new JLabel("Flows out    ");
		label.setFont(fontHeadings);
		con = new GridBagConstraints();
		con.gridx = 3;
		con.gridy = 0;
		gblayout.setConstraints(label,con);
		panelFlows.add(label);

		label = new JLabel("Ratio name ");
		label.setFont(fontHeadings);
		con.gridx = 4;
		gblayout.setConstraints(label,con);
		panelFlows.add(label);

		label = new JLabel("Prescribed ");
		label.setFont(fontHeadings);
		con.gridx = 5;
		gblayout.setConstraints(label,con);
		panelFlows.add(label);

		label = new JLabel("     ");
		con.gridx = 6;
		gblayout.setConstraints(label,con);
		panelFlows.add(label);

		// Flow names, ratio names and checkboxes for flow ratios
		con = new GridBagConstraints();
		int iflowratio = 0;
		iter =vrflowratio.iterator();
		while (iter.hasNext()) {
			FlowRatio flowratio = (FlowRatio)iter.next();
			// Name
			label = new JLabel(flowratio.getFlowOut().getName());
			con.gridx = 3;
			con.gridy = iflowratio+1;
			gblayout.setConstraints(label,con);
			panelFlows.add(label);

			// Ratio name
			vrtxtfldRatio[iflowratio] = new JTextField(10);
			vrtxtfldRatio[iflowratio].setText(flowratio.getName());
			vrtxtfldRatio[iflowratio].setEnabled(true);
			con.gridx = 4;
			con.gridy = iflowratio+1;
			gblayout.setConstraints(vrtxtfldRatio[iflowratio],con);
			panelFlows.add(vrtxtfldRatio[iflowratio]);

			// Checkbox
			vrchkboxPrescr[iflowratio] = new JCheckBox();
			// Check if flow ratio is prescribed
			if (flowratio.isPrescribed()) {
				// vrchkboxPrescr[iflowratio].setEnabled(true);
				vrchkboxPrescr[iflowratio].setSelected(true);
			} // if
			con.gridx = 5;
			con.gridy = iflowratio+1;
			gblayout.setConstraints(vrchkboxPrescr[iflowratio],con);
			panelFlows.add(vrchkboxPrescr[iflowratio]);

			// Make an import button
			vrbtnImport[iflowratio] = new JButton("Import data");
			con.gridx = 6;
			con.gridy = iflowratio+1;
			gblayout.setConstraints(vrbtnImport[iflowratio],con);
			panelFlows.add(vrbtnImport[iflowratio]);

			// Add focus listener
			vrtxtfldRatio[iflowratio].addFocusListener(this);
			// Add actionlisteners
			vrtxtfldRatio[iflowratio].addActionListener(this);
			vrchkboxPrescr[iflowratio].addActionListener(this);
			vrbtnImport[iflowratio].addActionListener(this);
			iflowratio++;
		} // while
		disableLastFlowRatio();

        return panelFlows;
	} // makeFlowsPanel

	/** Make a panel with a table of prescribed and calculated flow ratios.
	* @return A table panel.*/
	protected JPanel makeTablePanel() {

		// Create the table panel
		JPanel panelTable = new JPanel();
		panelTable.setLayout(new BorderLayout());

		// Make table
		try {
			tabelmodel = new SplitterTableModel(this,vrflowratio);
			JTable table = new JTable(tabelmodel);
			table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			// Create the scroll pane and add the table to it.
			JScrollPane scrollPane = new JScrollPane(table);
			panelTable.add(scrollPane,BorderLayout.CENTER);
			labelErrMsg = new JLabel(" ");
			labelErrMsg.setForeground(Color.red);
			panelTable.add(labelErrMsg,BorderLayout.SOUTH);
			panelTable.setPreferredSize(new Dimension(200,(int)(35*cflowratio+30)));
		} // vrflowratio.size()>0
		// else
		catch (Exception e) {
			// No flows out
			JPanel panelErrMsg = new JPanel();
			panelErrMsg.setLayout(new BoxLayout(panelErrMsg,BoxLayout.Y_AXIS));
			labelErrMsg = new JLabel("No flows out are known.");
			labelErrMsg.setForeground(Color.red);
			panelErrMsg.add(labelErrMsg);
			panelErrMsg.add(new JLabel("If there are flows out,"));
			panelErrMsg.add(new JLabel("try to disconnect them and reconnect them again."));
			panelTable.add(panelErrMsg);
		} // try/catch

		return panelTable;

	} // makeTablePanel

	protected JPanel makePlotPanel() {

		// Panel for painting flow plot
		JPanel panelPlot = new JPanel(new BorderLayout());
		// Make canvas for drawing flow ratio plot
		splittercanvas = new SplitterCanvas(getSize().width,splitter);
		panelPlot.add(splittercanvas,BorderLayout.CENTER);
        panelPlot.setBorder(BorderFactory.createTitledBorder(
        	"Flow ratios as a function of time"));

		return panelPlot;
	} // makePlotPanel

	protected JPanel makeButtonsPanel() {

		JPanel panelButtons = new JPanel(new FlowLayout());
		// Buttons
		btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		panelButtons.add(btnOK);
		return panelButtons;
	} // makeButtonsPanel

	public void showErrorMsg(String errmsg) {
		labelErrMsg.setText(errmsg);
	} // showErrorMsg

	/** Repaint ratio curves. */
	public void repaint() { if (splittercanvas!=null) splittercanvas.repaint(); }

	/** Invoked when a txtfldRatio gains the keyboard focus,
	* does nothing*/
	public void focusGained(FocusEvent e) {
	} // focusGained

	/** Invoked when a txtfldRatio loses the keyboard focus,
	* sets name of flow ratio. */
	public void focusLost(FocusEvent e) {
		// Ratio names, Check boxes
		for (int iflowratio=0; iflowratio<cflowratio; iflowratio++) {
			FlowRatio flowratio = (FlowRatio)vrflowratio.elementAt(iflowratio);
			if (e.getSource()==vrtxtfldRatio[iflowratio])    {
				flowratio.setName(vrtxtfldRatio[iflowratio].getText());
			} // if
		} // for
		if (tabelmodel!=null) tabelmodel.fireTableDataChanged();
	} // focusLost

	/** Respond to checkboxes and buttons. */
	public void actionPerformed(ActionEvent e) {

		// Ratio names, Check boxes, Import buttons
		for (int iflowratio=0; iflowratio<cflowratio; iflowratio++) {
			FlowRatio flowratio = (FlowRatio)vrflowratio.elementAt(iflowratio);
			if (e.getSource()==vrchkboxPrescr[iflowratio])    {
				if (vrchkboxPrescr[iflowratio].isSelected()) {
					flowratio.setPrescribed();
				} else {
					flowratio.setDefault();
				} // if/else
				disableLastFlowRatio();
			} // if
			if (e.getSource()==vrtxtfldRatio[iflowratio])    {
				flowratio.setName(vrtxtfldRatio[iflowratio].getText());
			} // if
			if (e.getSource()==vrbtnImport[iflowratio])    {
				// Import new data to flowratio with index iflowratio
				new ImportDialog(this, "Import splitter ratio",flowratio);
				tabelmodel.updateTable(iflowratio);
			} // if
		} // for
		if (tabelmodel!=null) tabelmodel.fireTableDataChanged();

    	if (e.getSource()==btnOK) {
			// Set name of splitter
			splitter.setName(txtfldName.getText());
			// Ratio names, Check boxes
			for (int iflowratio=0; iflowratio<cflowratio; iflowratio++) {
				FlowRatio flowratio = (FlowRatio)vrflowratio.elementAt(iflowratio);
				flowratio.setName(vrtxtfldRatio[iflowratio].getText());
			} // for
			dispose();
			Calculator.calculate();
		} // if btnOK

	} // actionPerformed

} // class SplitterDialog