import javax.swing.*;     // JFileChooser
import java.io.*;         // File
import java.util.*;		  // Date
import java.text.*;       // DateFormat

/** Exports model data to Text file.*/
public class TextExporter extends Exporter {

	public static String szCommandStart = "";
	public static String szCommentSign = "";

	/** Creates a TextExporter for writing model data to file.
	* @param szNameXMLFile The name of the XML model file.
	* @param szExtension File extension of exported file. */
	public TextExporter(String szNameXMLFile, String szExtension) {
		super(szNameXMLFile,szExtension);
	} // TextExporter

	/** List at selected time. */
	public String listAtSelectedTime() {

		// Selected time
		String szExport = "\n\n" + szCommentSign + "Model time: " +
			timeaxis.getSelectedTime() + ";\n" +
			szCommentSign + "=======================================;";

		// List for layers
		Vector vrlayer = ECF.getLayers();
		Iterator iter = vrlayer.iterator();
		int ilayer = 0;
		// Flows
		while (iter.hasNext()) {
			Layer layer = (Layer)iter.next();
			szExport += "\n\nLayer " + layer.getName()+
				"\n=============================================\n\n";

			szExport += "\nPRESCRIBED variables\n====================\n";
			szExport += listVariableAtTimeWithState(vrflow,"Flow", "flow",
					szCommentSign,indexOffset(),ilayer,Variable.PRESCRIBED);
			szExport += listVariableAtTimeWithState(vrflowratio,"FlowRatio", "flow ratio",
					szCommentSign,indexOffset(),ilayer,Variable.PRESCRIBED);
			szExport += listVariableAtTimeWithState(vrprice,"FlowPrice", "price",
					szCommentSign,indexOffset(),ilayer,Variable.PRESCRIBED);
			szExport += listVariableAtTimeWithState(vrproductivity,"Productivity", "productivitie",
					szCommentSign,indexOffset(),ilayer,Variable.PRESCRIBED);

			szExport += "\nCALCULATED variables\n====================\n";
			szExport += listVariableAtTimeWithState(vrflow,"Flow", "flow",
					szCommentSign,indexOffset(),ilayer,Variable.CALCULATED);
			szExport += listVariableAtTimeWithState(vrflowratio,"FlowRatio", "flow ratio",
					szCommentSign,indexOffset(),ilayer,Variable.CALCULATED);
			szExport += listVariableAtTimeWithState(vrprice,"FlowPrice", "price",
					szCommentSign,indexOffset(),ilayer,Variable.CALCULATED);
			szExport += listVariableAtTimeWithState(vrproductivity,"Productivity", "productivitie",
					szCommentSign,indexOffset(),ilayer,Variable.CALCULATED);

			szExport += "\nDEFAULT variables\n=================\n";
			szExport += listVariableAtTimeWithState(vrflow,"Flow", "flow",
					szCommentSign,indexOffset(),ilayer,Variable.DEFAULT);
			szExport += listVariableAtTimeWithState(vrflowratio,"FlowRatio", "flow ratio",
					szCommentSign,indexOffset(),ilayer,Variable.DEFAULT);
			szExport += listVariableAtTimeWithState(vrprice,"FlowPrice", "price",
					szCommentSign,indexOffset(),ilayer,Variable.DEFAULT);
			szExport += listVariableAtTimeWithState(vrproductivity,"Productivity", "productivitie",
					szCommentSign,indexOffset(),ilayer,Variable.DEFAULT);

			ilayer++;
		} // while
		return szExport;
	} // listAtSelectedTime

	/** List the time histories of variables.
	* @return A string listing time histories.*/
	public String listTimeHistories() {
		String szExport = "";
		// Time axis
		if (timeaxis!=null)	szExport += "     " +timeaxis.export();
		// Data vectors
		szExport += listFlowTimeHistories();
		szExport += listFlowRatioTimeHistories();
		szExport += listPriceTimeHistories();
		szExport += listProductivityTimeHistories();
		return szExport;
	} // listTimeHistories

	/** List flow time histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each flow line. */
	protected String listFlowTimeHistories() {
		// Flows as a function of time
		String szFlows = "";
		if (vrflow==null)	return szFlows;
		int iflow = 1;
		Iterator iter = vrflow.iterator();
		while (iter.hasNext()) {
			String szIFlow = ECFUtil.padCharacters(iflow,3,' ');
			szFlows += szIFlow + ": " + ((Flow)iter.next()).export();
			iflow++;
		} // while
		return szFlows;
	} // listFlowTimeHistories

	/** List flow ratio time histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each flow ratio line. */
	protected String listFlowRatioTimeHistories() {
		// Flow ratios as a function of time
		String szFlowRatios = "";
		if (vrflowratio==null)	return szFlowRatios;
		int iratio = 1;
		Iterator iter = vrflowratio.iterator();
		while (iter.hasNext()) {
			String szIRatio = ECFUtil.padCharacters(iratio,3,' ');
			szFlowRatios += szIRatio + ": " + ((FlowRatio)iter.next()).export();
			iratio++;
		} // while
		return szFlowRatios;
	} // listFlowTimeHistories

	/** List price histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each price line. */
	protected String listPriceTimeHistories() {
		// Prices as a function of time
		String szPrices = "";
		if (vrprice==null)	return szPrices;
		int iprice = 1;
		Iterator iter = vrprice.iterator();
		while (iter.hasNext()) {
			String szIPrice = ECFUtil.padCharacters(iprice,3,' ');
			szPrices += szIPrice + ": " + ((Price)iter.next()).export();
			iprice++;
		} // while
		return szPrices;
	} // listPriceTimeHistories

	/** List productivity histories.
	* @return A string with semicolon(;) separates values and line feeds
	* between each productivity line. */
	protected String listProductivityTimeHistories() {
		// Productivities as a function of time
		String szProductivities = "";
		if (vrproductivity==null)	return szProductivities;
		int iproductivity = 1;
		Iterator iter = vrproductivity.iterator();
		while (iter.hasNext()) {
			String szIProductivity = ECFUtil.padCharacters(iproductivity,3,' ');
			szProductivities += szIProductivity + ": " + ((Productivity)iter.next()).export();
			iproductivity++;
		} // while
		return szProductivities;
	} // listProductivityTimeHistories

} // TextExporter