import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class TimeAxisDialog extends JDialog implements ActionListener {

	private TimeAxis timeaxisIn;  // Original axis from input
	private TimeAxis timeaxis;    // Copy displayed and modified

	private Container panelContent;
	private JPanel panelxisControl = new JPanel();
	private GridBagLayout gridbaglayout = new GridBagLayout();
	private GridBagConstraints gbcon = new GridBagConstraints();

	private JTextField textStartTime;
	private JTextField textStart;
	private JTextField textEnd;
	private JButton btnExtend;
	private JTextField textFirstCase;
	private JButton btnRenumber;
	private JButton btnNew;
	private JButton btnSetTime;
	private JTextField textStartOfIncrement;
	private JTextField textLengthOfIncrement;
	private JButton btnSetIncrement;
	private JLabel labelWarning;

	private TimeAxisCanvas canvas;

	private JButton btnOK;
	private JButton btnCancel;

	public TimeAxisDialog() {
		super(ECF.frameMain, "Time Axis", true);
		timeaxisIn = ECF.getTimeAxis();
		timeaxis = (TimeAxis)timeaxisIn.clone();
		try {
			Point location = ECF.frameMain.getLocationOnScreen();
			location.translate(20,10);
			setLocation(location);
		} catch(Exception e) {
			// ECFMainFram did not finish.
			return;
		} // try/catch
		setSize(500,320);

		panelContent = getContentPane();
		try {makeContentPane();}
		catch(Exception e) {System.out.println(e);}
		pack();
		show();

	} // TimeAxisDialog

	protected void makeContentPane() {

		// Axis control panel
		panelxisControl.setLayout(gridbaglayout);
		gbcon.gridy = 0;
		gbcon.anchor = GridBagConstraints.WEST;
		panelContent.add(panelxisControl,BorderLayout.NORTH);

		int status = timeaxis.getStatus();
		if (status==TimeAxis.EMPTY || status==TimeAxis.NEW) {
			// New axis panel
			makeNewAxisPanel();
			// Set increment panel
			makeSetIncrementPanel();
		} else if (status==TimeAxis.SINGLE_NO_TIME_INCREMENT) {
			// Set time panel
			makeStartTimePanel();
			// Extend panel
			makeExtendPanel();
			// Set increment panel
			makeSetIncrementPanel();
		} else if (status==TimeAxis.EXTENDED|| status==TimeAxis.LOCKED) {
			// Extend panel
			makeExtendPanel();
			// Set increment panel
			makeSetIncrementPanel();
			// Renumbering panel
			makeRenumberingPanel();
		} // else/if

		// Warning label
		makeWarningLabel();

		// Canvas
		canvas = new TimeAxisCanvas(getSize().width, timeaxis);
		panelContent.add(canvas, BorderLayout.CENTER);

		// OK and Cancel buttons
		btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(this);
		JPanel panelButtons = new JPanel();
		panelButtons.setLayout(new FlowLayout());
		panelButtons.add(btnOK);
		panelButtons.add(btnCancel);
		panelContent.add(panelButtons, BorderLayout.SOUTH);
	} // makeContentPane

	private void makeExtendPanel() {
		// Label start time
		JLabel labelStart = new JLabel("Start Time");
		gbcon.gridx = 0;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelStart,gbcon);
		panelxisControl.add(labelStart);
		// Textfield start time
		textStart = new JTextField(7);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,10);
		gridbaglayout.setConstraints(textStart,gbcon);
		panelxisControl.add (textStart);
		// Label end time
		JLabel labelEnd = new JLabel("End Time");
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelEnd,gbcon);
		panelxisControl.add (labelEnd);
		// Textfield end time
		textEnd = new JTextField(7);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,10);
		gridbaglayout.setConstraints(textEnd,gbcon);
		panelxisControl.add (textEnd);
		// Extend axis button
		btnExtend = new JButton("Extend Axis");
		btnExtend.addActionListener(this);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(btnExtend,gbcon);
		panelxisControl.add(btnExtend);
		// Step to next line of inputs
		gbcon.gridy++;
	} // makeExtendPanel

	private void makeRenumberingPanel() {
		// Label start time
		JLabel labelStart = new JLabel("First Case");
		gbcon.gridx = 0;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelStart,gbcon);
		panelxisControl.add(labelStart);
		// Textfield start case
		textFirstCase = new JTextField(7);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,10);
		gridbaglayout.setConstraints(textFirstCase,gbcon);
		panelxisControl.add (textFirstCase);
		// Label renumbering
		JLabel labelRenumber = new JLabel("Cases will be renumbered with unit increments.");
		gbcon.gridx++;
		gbcon.gridwidth = 2;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelRenumber,gbcon);
		panelxisControl.add (labelRenumber);
		gbcon.gridx++;
		// Renumber button
		btnRenumber = new JButton("Renumber as Cases");
		btnRenumber.addActionListener(this);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(btnRenumber,gbcon);
		panelxisControl.add(btnRenumber);
		// Step to next line of inputs
		gbcon.gridy++;
	} // makeRenumberingPanel

	private void makeNewAxisPanel() {
		// Label start time
		JLabel labelStart = new JLabel("Start Time");
		gbcon.gridx = 0;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelStart,gbcon);
		panelxisControl.add(labelStart);
		// Textfield start time
		textStart = new JTextField(7);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,10);
		gridbaglayout.setConstraints(textStart,gbcon);
		panelxisControl.add (textStart);
		// Label end time
		JLabel labelEnd = new JLabel("End Time");
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelEnd,gbcon);
		panelxisControl.add (labelEnd);
		// Textfield end time
		textEnd = new JTextField(7);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,10);
		gridbaglayout.setConstraints(textEnd,gbcon);
		panelxisControl.add (textEnd);
		// New axis button
		btnNew = new JButton("Make time axis");
		btnNew.addActionListener(this);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(btnNew,gbcon);
		panelxisControl.add(btnNew);
		// Step to next line of inputs
		gbcon.gridy++;
	} // makeNewAxisPanel

	private void makeSetIncrementPanel() {
		// Label start time of increment
		JLabel labelStartOfIncrement = new JLabel("Time of increment");
		gbcon.gridx = 0;
		gridbaglayout.setConstraints(labelStartOfIncrement,gbcon);
		panelxisControl.add(labelStartOfIncrement);
		// Textfield start time
		textStartOfIncrement = new JTextField(7);
		gbcon.gridx++;
		gridbaglayout.setConstraints(textStartOfIncrement,gbcon);
		panelxisControl.add (textStartOfIncrement);
		// Label length of increment
		JLabel labelLengthOfIncrement = new JLabel("Length of increment");
		gbcon.gridx++;
		gridbaglayout.setConstraints(labelLengthOfIncrement,gbcon);
		panelxisControl.add (labelLengthOfIncrement);
		// Label length of increment
		textLengthOfIncrement = new JTextField(7);
		gbcon.gridx++;
		gridbaglayout.setConstraints(textLengthOfIncrement,gbcon);
		panelxisControl.add (textLengthOfIncrement);
		// Button set increment
		btnSetIncrement = new JButton("Set time increment");
		btnSetIncrement.addActionListener(this);
		gbcon.gridx++;
		gridbaglayout.setConstraints(btnSetIncrement,gbcon);
		panelxisControl.add(btnSetIncrement);
		// Step to next line of inputs
		gbcon.gridy++;
	} // makeSetIncrementPanel

	private void makeStartTimePanel() {
		// New axis  panel
		// Label start time
		JLabel labelStart = new JLabel("Start time");
		gbcon.gridx = 0;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(labelStart,gbcon);
		panelxisControl.add(labelStart);
		// Textfield start time
		textStartTime = new JTextField(7);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,10);
		gridbaglayout.setConstraints(textStartTime,gbcon);
		panelxisControl.add (textStartTime);
		// Set time button
		btnSetTime = new JButton("Set time");
		btnSetTime.addActionListener(this);
		gbcon.gridx++;
		gbcon.insets = new Insets(0,0,0,4);
		gridbaglayout.setConstraints(btnSetTime,gbcon);
		panelxisControl.add(btnSetTime);
		// Step to next line of inputs
		gbcon.gridy++;
	} // makeStartTimePanel

	private void makeWarningLabel() {
		labelWarning = new JLabel("");
		labelWarning.setForeground(Color.red);
		gbcon.gridx = 0;
		gbcon.gridwidth = 5;
		gridbaglayout.setConstraints(labelWarning,gbcon);
		panelxisControl.add(labelWarning);
		// Step to next line of inputs
		gbcon.gridy++;
	} // makeWarningLabel

	public void actionPerformed(ActionEvent e1) {

		if (e1.getSource()==btnSetTime) {
			try {
				timeaxis.setStartTime(textToDouble(textStartTime));
				canvas.repaint();
			} catch(Exception e2) {;}
		} else 		if (e1.getSource()==btnNew) {
			try {
				timeaxis.makeNew(textToDouble(textStart),textToDouble(textEnd));
				canvas.repaint();
			} catch(Exception e2) {;}
		} else if (e1.getSource()==btnExtend) {
			try {
				timeaxis.extend(textToDouble(textStart),textToDouble(textEnd));
				if (textStartTime!=null) textStartTime.setEnabled(false);
				if (btnSetTime!=null) btnSetTime.setEnabled(false);
				canvas.repaint();
			} catch(Exception e2) { System.out.println("TimeAxisDialog.actionPerformed: extend \n" + e2);}
		} else if (e1.getSource()==btnSetIncrement) {
			try {
				timeaxis.setIncrementTime(textToDouble(textStartOfIncrement),textToDouble(textLengthOfIncrement));
				canvas.repaint();
			} catch(Exception e2) {;}
		} else if (e1.getSource()==btnRenumber) {
			try {
				timeaxis.renumber(textToDouble(textFirstCase));
				if (textStartTime!=null) textStartTime.setEnabled(false);
				if (btnSetTime!=null) btnSetTime.setEnabled(false);
				canvas.repaint();
			} catch(Exception e2) { System.out.println("TimeAxisDialog.actionPerformed: renumber \n" + e2);}
		} else if (e1.getSource()==btnOK) {
			timeaxisIn.setStates(timeaxis); // Copy temporary time axis into original time axis
			timeaxisIn.adjustVariablesToTimeAxis();
			timeaxisIn.lock();
			dispose();
			ECFModel.fModelWasChanged = true;
		} else if(e1.getSource()==btnCancel) {
			dispose();
		} // if

	} // actionPerformed

	private double textToDouble(JTextField textfield) throws Exception {
		String szTime = textfield.getText();
		double time = 0.0;
		if (szTime.length()>0) {
			try {
				time = Double.parseDouble(szTime);
				textfield.setForeground(Color.black);
				labelWarning.setText("");
			} catch(Exception e) {
				textfield.setForeground(Color.red);
				labelWarning.setText("Not a number format");
				throw e;
			} // try/catch
		} // if
		return time;
	} // textToDouble

} // TimeAxisDialog

