/** Time increment keeps information about a time interval
* starting at a given time. */
public class TimeIncrement {

	/** Time at start of the time increment. */
	protected double time = 0.0;
	/** Description of the time increment. */
	protected String szDescription = "Description";

	/** Create a time increment with start time = 0.0 and no descrioption. */
	public TimeIncrement() {
	} // TimeIncrement

	/** Create a time increment with no descrioption.
	* @param time The start time of the time increment. */
	public TimeIncrement(double time) {
		this.time = time;
	} // TimeIncrement

	/** Make a clone of the time axis = a copy with its own data. */
	public Object clone() {
		TimeIncrement timeinc = new TimeIncrement();
		timeinc.time = time;
		timeinc.szDescription = szDescription;
		return (Object)timeinc;
	} // clone

	/** Decode the time increment from XML code.
	* @param szXMLCode The XML code for the time increment. */
	public void decodeXML(String szXMLCode) {
//		System.out.println("TimeIncrement.decodeXML: szXMLCode='" + szXMLCode + "'");
		XMLParser xml = null;
		// Decode XML code
		try {
			while (szXMLCode.length()>0) {
				xml = new XMLParser(szXMLCode);
				String szTag = xml.tag();
				String szSub = xml.sub();
				if (szTag.equals("time")) time = (new Double(szSub)).doubleValue();
				if (szTag.equals("szDescription"))  szDescription = szSub;
				szXMLCode = xml.remainder();
			} // while
		} catch(Exception e) {
			System.out.println(e);
		} // try/catch
//		System.out.println("TimeIncrement.decodeXML: " + this);
	} // decodeXML

	/** Get description.
	* @return Description of the time increment. */
	public String getDescription() { return szDescription; }

	/** Get start time.
	* @return Start time. */
	public double getStartTime() { return time; }

	/** List XML code for this time increment.
	* @return XML code. */
	public String listXML(String szBlanks) {
		String szXML = "";
		// Start time
		szXML += "<time>" + time + "</time>";
		szXML += "<szDescription>" + szDescription + "</szDescription>";
		return szXML;
	} // listXML

	/** Set description.
	* @param szDescription Description of the time increment. */
	public void setDescription(String szDescription)
		{ this.szDescription = szDescription; }

	/** Set start time.
	* @param time Start time. */
	public void setStartTime(double time) { this.time = time; }

	/** Returns a string representation of the time increment.
	* @return A string representation of the time axis.*/
    public String toString() {
		return "TimeIncrement: time=" + time +
			"\nszDescription='" + szDescription + "'";
	} // toString

} // TimeIncrement