import java.awt.*;        // FlowLayout
import java.util.*;       // Vector
import javax.swing.*;     // JPanel
import javax.swing.border.BevelBorder;
import java.awt.event.*;  // AdjustmentListener

/** The time scroll panel has <br>
* - a label "Time:",<br>
* - a text field shoving curenbt time,<br>
* - a scroll bar for scrolling between times.*/
public class TimeScrollPanel extends JPanel implements AdjustmentListener {

	/** Panel showing alert warning and time increment description. */
	protected AlertPanel panelAlert;
	/** The model graphs panel. */
	protected GraphsPanel panelGraphs;
	/** */
	private   JTextField textfieldTime;
	/** */
	private   JScrollBar scrollbarTime;

	/** Creates a time scroll panel.
	* @param panelAlert The alert panel.
	* @param panelGraphs The graphs panel. */
	public TimeScrollPanel(AlertPanel panelAlert, GraphsPanel panelGraphs) {
		this.panelAlert = panelAlert;
		this.panelGraphs = panelGraphs;
		createControls();
	} // TimeScrollPanel

	/** Creates the controls on the time scroll panel. */
	protected void createControls() {

		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Time label
		JPanel panelTime = new JPanel();
		panelTime.setLayout(new FlowLayout(FlowLayout.LEFT));
		panelTime.add(new JLabel("Time:   "));

		// Text field for time
		textfieldTime = new JTextField(7);
		textfieldTime.setEditable(false);
		panelTime.add(textfieldTime);
		panelTime.setSize(20,100);
		add(panelTime);

		// Scrollbar for time
		scrollbarTime = new JScrollBar(JScrollBar.HORIZONTAL,
										0,  // value
										1,  // extent
										0,  // min
										1); // max
		scrollbarTime.setUnitIncrement(1);
		scrollbarTime.setMaximum(ECF.timeaxis.getTimeIncrements().size()-1);
		scrollbarTime.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		scrollbarTime.addAdjustmentListener(this);
		scrollbarTime.setSize(30,700);
        add(scrollbarTime);

		setTimeInTextField();

	} // createControls

	/** Responds to scrollbar adjustment */
	public void adjustmentValueChanged(AdjustmentEvent e) {

		// Set selected increment of time axis
		ECF.timeaxis.setSelectedIncrement(scrollbarTime.getValue());
		// Update text in alert panel
		panelAlert.update();
		// Set time value in text field
		setTimeInTextField();

		Calculator.calculate();
		panelGraphs.repaintAll();

	} // adjustmentValueChanged

	/** Initialize maximum and current values of scroll bar.
	* @param fOpenTimeAxisDialog = true if TimeAxis dialog should be opened, else = false.
	*/
	public void initSettings() {
		// Init scroll bar.
		scrollbarTime.removeAdjustmentListener(this);
		scrollbarTime.setMaximum(ECF.timeaxis.getSize()-1);
		scrollbarTime.setVisibleAmount(1);
		scrollbarTime.setValue(ECF.timeaxis.getSelectedIncrement());
		scrollbarTime.addAdjustmentListener(this);
		// Set time value in text field
		setTimeInTextField();
		// Clear text in alert panel
//		panelAlert.clear();
	} // initSettings

	/** */
	private void setTimeInTextField() {
		// Set time value in text field
		Vector vrtimeInc = ECF.timeaxis.getTimeIncrements();
		int iincSelected = scrollbarTime.getValue();
		double szTime = ((TimeIncrement)vrtimeInc.elementAt(iincSelected)).getStartTime();
		textfieldTime.setText(String.valueOf(szTime));
	} // setTimeInTextField

} // class TimeScrollPanel