/**
* A unit for a Quantity with conversion factor. <br>
* <code> Unit unitCU = new Unit('CU',1.0,'Currency base unit'); </code> <br>
* <code> Unit unitkCU = new Unit('kCU',0.001,'Thousand CU'); </code> <br>
*/
public class Unit {

	/** The short name of the unit */
	String szName;
	/** Conversion factor from unit to base unit. */
	double factor;
	/** Long description of unit. */
	String szDescription;

	/** Creates a unit with name <code>szName</code> and conversion factor
	* <code>factor</code>.
	* A number times a unit = a <a href="Quantity.html">Quantity<a/>.
	* @param szName The short name of the unit
	* @param factor Conversion factor from unit to base unit.
	* @param szDescription Long description of unit.
	*/
	public Unit(String szName, double factor,
		String szDescription) {
		this.szName = szName;
		this.factor = factor;
		this.szDescription = szDescription;
	} // Unit

	/** Creates a unit with conversion factor = 1 for a Quantity.
	*/
	public Unit() {
		this("name",1.0,"description");
	} // Unit

	/** Make a copy of the unit.
	@ return A copy (clone). */
	public Object clone() {
		Unit clone = new Unit();
		clone.szName = szName;
		clone.factor = factor;
		clone.szDescription = szDescription;
		return clone;
	} // clone

	/**
	* Decode unit from XML code.
	* @param szXMLCode The xml description of this unit.
	*/
	public void decodeXML(String szXMLCode) {
		XMLParser xml = null;
		// Decode XML code
		try {
			while (szXMLCode.length()>0) {
			xml = new XMLParser(szXMLCode);
				String szTag = xml.tag();
				String szSub = xml.sub();
				if (szTag.equals("szName")) szName = szSub;
				if (szTag.equals("factor")) factor = Double.parseDouble(szSub);
				if (szTag.equals("szDescription")) szDescription = szSub;
				szXMLCode = xml.remainder();
			} // while
		} catch(Exception e) {
		} // try/catch
	} // decodeXML

	/** Get the name of the unit.
	* @return The name of the unit. */
	public String getName() { return szName; }
	/** Get the conversion factor.
	* @return Conversion factor from unit to base unit. */
	public double getFactor() { return factor; }
	/** Get the description of the unit.
	* @return Long description of unit. */
	public String getDescription() { return szDescription; }

	/** List XML code for this unit.
	* @return XML code. */
	public String listXML(String szBlanks1) {
		String szBlanks = ""; // To give one compact line
		String szXML = "";
		szXML += szBlanks +"<szName>"+szName+"</szName>";
		szXML += "<factor>"+factor+"</factor>";
		szXML += "<szDescription>"+szDescription+"</szDescription>";
		return szXML;
	} // listXML

	/** String representation of unit.
	* @return The string representation.*/
	public String toString() {
		return "Unit: " + szName + " f=" + factor + " " + szDescription;
	}
} // Unit