import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** The units dialog is used for selecting display unit for a quantity,
* setting maximum value for flow arrows and assigning units to layers.
* <p>
* A unit can only be assigned to one layer. The <code>Clear layer units</code>
* buttons remove all units assignments for one layer. Units ssignments for a
* quantity can not be chanded before the previous assignment is removed.
* <p>
* <img src="UnitsDialog.gif"> */
public class UnitsDialog extends JDialog implements ActionListener, KeyListener {

	/** All defined sets of units. */
	protected Vector vrunits;
	/** Number of model layers .*/
	protected int clayer;

	/** Clear units button. */
	private JButton[] vrbtnClearLayerUnits;
	/** Combo boxes for setting units.*/
	private JComboBox[] vrcombo;
	/** Text fields for setting max values for painting flow arrows. */
	private JTextField[] vrtxtfdMax;
	/** Text fields for for layer index assignments. */
	private JTextField[] vrtxtfdLayerIndex;
	/** Accept settings and close dilaog button*/
	private JButton btnOK;
	/** Warning message label. */
	private JLabel labelMsg;

	/** Creates a units dialog. */
	public UnitsDialog() {
		super(ECF.frameMain, "Units", true); // Create as modal dialog
		try {
			Point location = ECF.frameMain.getLocationOnScreen();
			location.translate(20,10);
			setLocation(location);
		} catch(Exception e) {
			// ECFMainFrame did not finish.
			return;
		} // try/catch

		// Get common data.
		vrunits = ECF.getUnits();
		clayer = ECF.getLayers().size();
		vrcombo = new JComboBox[vrunits.size()];
		vrtxtfdMax = new JTextField[vrunits.size()];
		vrtxtfdLayerIndex = new JTextField[vrunits.size()];

		Container pane = getContentPane();

		// Layer panel
		JPanel panelLayers = makeLayerPanel(ECF.getLayers());
		pane.add(panelLayers,BorderLayout.NORTH);

		// Units panel
		JPanel panelUnits = makeUnitsPanel(vrunits);
		pane.add(panelUnits,BorderLayout.CENTER);

		// Buttons panel
		JPanel panelButtons = new JPanel();
		btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		panelButtons.setLayout(new FlowLayout());
		panelButtons.add(btnOK);
		pane.add(panelButtons, BorderLayout.SOUTH);

		pack();
		show();

	} // UnitsDialog

	private JPanel makeLayerPanel(Vector vrlayer) {
		GridBagLayout layoutGB = new GridBagLayout();
		JPanel panelLayers = new JPanel(layoutGB);
		GridBagConstraints constrGB = new GridBagConstraints();
		constrGB.gridy = 0;
		vrbtnClearLayerUnits = new JButton[clayer];

		// Show layers
		Iterator iter = vrlayer.iterator();
		int ilayer = 0;
		while (iter.hasNext()) {
			// Show one layer
			Layer layer = (Layer)iter.next();

			// Label layer index
			JLabel labelIndex = new JLabel("Layer #" + (ilayer+1));
			constrGB.gridx = 0;
			constrGB.ipadx = 10;
			constrGB.anchor = GridBagConstraints.WEST;
			constrGB.insets = new Insets(0,0,0,0);
			layoutGB.setConstraints(labelIndex,constrGB);
			panelLayers.add(labelIndex);
			// Label layer name
			JLabel labelName = new JLabel(layer.getName());
			constrGB.gridx = 1;
			constrGB.anchor = GridBagConstraints.WEST;
			constrGB.insets = new Insets(0,0,0,0);
			layoutGB.setConstraints(labelName,constrGB);
			panelLayers.add(labelName);
			// Button
			vrbtnClearLayerUnits[ilayer] = new JButton("Clear layer units");
			vrbtnClearLayerUnits[ilayer].addActionListener(this);
			constrGB.gridx = 2;
			constrGB.anchor = GridBagConstraints.WEST;
			constrGB.insets = new Insets(0,0,0,0);
			layoutGB.setConstraints(vrbtnClearLayerUnits[ilayer],constrGB);
			panelLayers.add(vrbtnClearLayerUnits[ilayer]);

			constrGB.gridy++;
			ilayer++;
		} // while

		return panelLayers;
	} // makeLayerPanel

	private JPanel makeUnitsPanel(Vector vrunits) {

		// Units panel
		GridBagLayout layoutGB = new GridBagLayout();
		JPanel panelUnits = new JPanel(layoutGB);
		GridBagConstraints constrGB = new GridBagConstraints();
		constrGB.gridy = 0;

		// Show Units
		Iterator iter = vrunits.iterator();
		int iunits = 0;
		while (iter.hasNext()) {
			// Show one set of units
			Units units = (Units)iter.next();
			// Label
			JLabel label = new JLabel(units.getName());
			constrGB.gridx = 0;
			constrGB.ipadx = 10;
			constrGB.anchor = GridBagConstraints.WEST;
			constrGB.insets = new Insets(0,0,0,0);
			layoutGB.setConstraints(label,constrGB);
			panelUnits.add(label);

			// Combobox
			Vector vrComboItem = new Vector();
			Iterator iterUnit = units.iterator();
			while (iterUnit.hasNext()) {
				Unit unit = (Unit)iterUnit.next();
				vrComboItem.add(unit.getName());
			} // while
			vrcombo[iunits] = new JComboBox(vrComboItem);
			vrcombo[iunits].setSelectedIndex(units.getIndexOfDisplayUnit());
			vrcombo[iunits].addActionListener(this) ;
			constrGB.gridx = 1;
			constrGB.ipadx = 10;
			constrGB.fill = GridBagConstraints.HORIZONTAL;
			constrGB.anchor = GridBagConstraints.WEST;
			layoutGB.setConstraints(vrcombo[iunits],constrGB);
			panelUnits.add(vrcombo[iunits]);

			// Max scale label
			JLabel labelMax = new JLabel("  Max: ");
			constrGB.gridx = 2;
			layoutGB.setConstraints(labelMax,constrGB);
			panelUnits.add(labelMax);
			// Max scale value
			vrtxtfdMax[iunits] = new JTextField(10);
			vrtxtfdMax[iunits].setText(units.getStringMaxValue());
			vrtxtfdMax[iunits].addActionListener(this);
			constrGB.gridx = 3;
			constrGB.ipadx = 0;
			layoutGB.setConstraints(vrtxtfdMax[iunits],constrGB);
			panelUnits.add(vrtxtfdMax[iunits]);

			// Layer index label
			JLabel labelLayerIndex = new JLabel("  Layer #");
			constrGB.gridx = 4;
			layoutGB.setConstraints(labelLayerIndex,constrGB);
			panelUnits.add(labelLayerIndex);
			// Layer index value
			vrtxtfdLayerIndex[iunits] = new JTextField(1);
			int ilayer = units.getLayerIndex();
			String szLayerIndex;
			if(ilayer>=0)  {
				szLayerIndex = String.valueOf(1+units.getLayerIndex());
				vrtxtfdLayerIndex[iunits].setEnabled(false);
			}
			else szLayerIndex = " ";
			vrtxtfdLayerIndex[iunits].setText(szLayerIndex);
			vrtxtfdLayerIndex[iunits].addKeyListener(this);
			constrGB.gridx = 5;
			constrGB.ipadx = 0;
			constrGB.insets = new Insets(0,3,0,5);
			layoutGB.setConstraints(vrtxtfdLayerIndex[iunits],constrGB);
			panelUnits.add(vrtxtfdLayerIndex[iunits]);

			iunits++;
			constrGB.gridy++;
		} // while

		// Message label;
		labelMsg = new JLabel();
		labelMsg.setText(" ");
		constrGB.gridx = 0;
		constrGB.gridwidth = GridBagConstraints.REMAINDER;
		constrGB.gridy++;
		layoutGB.setConstraints(labelMsg,constrGB);
		panelUnits.add(labelMsg);

		return panelUnits;

	} // makeUnitsPanel

	/** Responds to <br>
	* - clear layer buttons<br>
	* - combo boxes<br>
	* - OK button */
	public void actionPerformed(ActionEvent e) {
		Iterator iter;
		Units units;
		int iunits;

		// Layer clear buttons
		for (int ilayer=0; ilayer<clayer; ilayer++) {
			if (e.getSource()==vrbtnClearLayerUnits[ilayer]) {
				iter = vrunits.iterator();
				iunits = 0;
				while (iter.hasNext()) {
					// One set of units
					units = (Units)iter.next();
					if (ilayer==units.getLayerIndex()) {
						units.setLayerIndex(-1);
						vrtxtfdLayerIndex[iunits].setText(" ");
						vrtxtfdLayerIndex[iunits].setEnabled(true);
						ECF.clearUnits(ilayer);
					} // if
				iunits++;
				} // while
			} // if
		} // for

		// Combo boxes
		iter = vrunits.iterator();
		iunits = 0;
		while (iter.hasNext()) {
			// One set of units
			units = (Units)iter.next();
			// Display unit
			if (e.getSource()==vrcombo[iunits]) {
				boolean fOK = parseMaxText(units,iunits);
				if (!fOK) return;
				units.setDisplayUnit((String)vrcombo[iunits].getSelectedItem());
				vrtxtfdMax[iunits].setText(units.getStringMaxValue());
			}  // Combo box event
			iunits++;
		} // while

		// OK button
		if (e.getSource()==btnOK) {
			iter = vrunits.iterator();
			iunits = 0;
			while (iter.hasNext()) {
				// One set of units
				units = (Units)iter.next();
				// Display unit
				units.setDisplayUnit((String)vrcombo[iunits].getSelectedItem());
				// Max scale
				boolean fOK = parseMaxText(units,iunits);
				if (!fOK) return;
				fOK = parseLayerIndex(units,iunits);
				if (!fOK) return;

				iunits++;
			} // while
			dispose();
		} // OK button

	} // actionPerformed

	/** Parse the text for max value and set max scale.
	* @return true if correct number format, else false.*/
	private boolean parseMaxText(Units units, int iunits) {
		try {
			double valueMax = Double.parseDouble(vrtxtfdMax[iunits].getText());
			units.setScaleToPixelsMax(valueMax);
			units.setScaleForCalc(valueMax);
			labelMsg.setText(" ");
		} catch (NumberFormatException excNF) {
			labelMsg.setText("Max value '" + excNF.getMessage() + "' has a bad format");
			return false;
		} catch (Exception exc) {
//			labelMsg.setText("Check for error: ");
			return false;
		}
		return true;
	} // parseMaxText

	/** Parse the text for layer index and set units layer index.
	* @return true if correct number format and range, else false.*/
	private boolean parseLayerIndex(Units units, int iunits) {
		try {
			String sz = (vrtxtfdLayerIndex[iunits].getText()).trim();
			if (sz.length()==0) return true; // Blank is a valid entry but gives no action
			int ilayer = Integer.parseInt(sz)-1;
			if (ilayer>=clayer) {
				labelMsg.setText("Layer index has to be in the range ' 1-" + clayer + "'");
				return false;
			} // if
			units.setLayerIndex(ilayer);
			labelMsg.setText(" ");
			ECF.setUnitsIfNotSet(ilayer,units);
			return true;
		} catch (NumberFormatException excNF) {
			labelMsg.setText("Layer index '" + excNF.getMessage() + "' has a bad format");
			return false;
		} catch (Exception exc) {
//			labelMsg.setText("Check for error: ");
			return false;
		}
		// return true;
	} // parseMaxText

	/** Responds to typing in the layer index field.
	* Checks that only an integer number 1-9 is typed*/
	public void keyTyped(KeyEvent e) {
		for (int iunits=0; iunits<vrtxtfdLayerIndex.length; iunits++) {
			if (e.getSource()==vrtxtfdLayerIndex[iunits]) {
				char ch = e.getKeyChar();
				String sz = "" + ch;
				// This leaves only one character in th text field
				vrtxtfdLayerIndex[iunits].setText("");
				try {
					int ilayer = Integer.parseInt(sz)-1;
					Units units = (Units)vrunits.elementAt(iunits);
					units.setLayerIndex(ilayer);
				} catch(NumberFormatException excNF) { ; }
			} // if
		} // for
	} // keyTyped

	/** Not used. */
	public void keyReleased(KeyEvent e) {
	} // keyReleased

	/** Not used. */
	public void keyPressed(KeyEvent e) {
	} // keyPressed

} // class UnitsDialog