import java.util.*; // Vector
import java.awt.*;  // Graphics
import java.awt.geom.AffineTransform;

/**
* The ValuesPlot class plots a stepvise plot of a <A href=Variable.html>Variable</A>.
* A plot looks like this:<p>
* <img src="ValuesPlot.gif">
*/

public class ValuesPlot {

	private double[] vrtime;
	private double[][] mxvalue;
	private Color vrcolor[];
	private int ctime;

	public ValuesPlot(Vector vrtimeInc, double[] vrvalue) {
		ctime = vrtimeInc.size();
		this.vrtime = new double[ctime];
		for (int itime=0; itime<ctime; itime++) {
			double time = ((TimeIncrement)vrtimeInc.elementAt(itime)).getStartTime();
			this.vrtime[itime] = time;
		} // for
		this.mxvalue = new double[1][];
		mxvalue[0] = vrvalue;
	} // ValuesPlot

	public ValuesPlot(Vector vrtimeInc, double[][] mxvalue, Color vrcolor[]) {
		ctime = vrtimeInc.size();
		this.vrtime = new double[ctime];
		for (int itime=0; itime<ctime; itime++) {
			double time = ((TimeIncrement)vrtimeInc.elementAt(itime)).getStartTime();
			this.vrtime[itime] = time;
		} // for
		this.mxvalue = mxvalue;
		this.vrcolor = vrcolor;
	} // ValuesPlot

	public void paint(Graphics g, int xOrigin, int yOrigin, int width, int height,
						double scaleValue, int iincSelected) {
		Graphics2D g2 = (Graphics2D)g;
		// Find time scale
		double timeMin = vrtime[0];
		double timeMax = vrtime[ctime-1];
		double timeTot = timeMax-timeMin;
		double scaleTime = width/(timeTot);

		// Plot values
		g2.translate(xOrigin,yOrigin);
		int cvalue = ctime-1;
		for (int icurve=0; icurve<mxvalue.length; icurve++) {
			g2.setColor(vrcolor[icurve]);
			for (int ivalue=0; ivalue<cvalue; ivalue++) {
				try {
				g2.drawLine((int)(scaleTime*(vrtime[ivalue]-timeMin)),
							(int)(-mxvalue[icurve][ivalue]*scaleValue),
				            (int)(scaleTime*(vrtime[ivalue+1]-timeMin)),
				            (int)(-mxvalue[icurve][ivalue]*scaleValue));
				int yend;
				if (ivalue<(cvalue-1)) yend = (int)(-mxvalue[icurve][ivalue+1]*scaleValue);
				else yend = 0;
				g2.drawLine((int)(scaleTime*(vrtime[ivalue+1]-timeMin)),
							(int)(-mxvalue[icurve][ivalue]*scaleValue),
				            (int)(scaleTime*(vrtime[ivalue+1]-timeMin)), yend);
			} catch (Exception ex) {
				System.out.println("ValuesPlot.paint: ivalue=" + ivalue + "\n" + ex);
			} // try/catch
			} // for
		} // for

		// Show selected increment by two vertical lines
		g2.setColor(Color.red);
		double timeLine = vrtime[iincSelected];
		int xLine = (int)(scaleTime*(timeLine-timeMin));
		g2.drawLine(xLine, 0, xLine, -height);
		timeLine = vrtime[iincSelected+1];
		xLine = (int)(scaleTime*(timeLine-timeMin));
		g2.drawLine(xLine, 0, xLine, -height);

		g2.setTransform(new AffineTransform());  // Reset to no translation
	} // paint

} // ValuesPlot