import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** The dialog set the properties of a Variable.*/
public class VariableDialog extends JDialog implements
	ActionListener,ItemListener {

	// Input parameters
	/** Location of upper left corner of the dilog. */
	protected Point location;
	/** The variable which properties shall be edited. */
	protected Variable variable;
	/** A vector of <a href="Units.html">Units</a> (unit sets).
	* One unit set that is assigned to to the same layer as the variable
	* can be chosen. */
	protected Vector vrunits;
	/** Check box legends. */
	protected String szCheckBoxPrescribedLegend = "Prescribed variable magnitude";
	protected String szCheckBoxMeasuredLegend = "Measured magnitude";
	/** Canvas legend. */
	protected String szCanvasLegend = "Variable as a function of time";
	/** The canvas for drawing the time history curves. */
	protected VariableCanvas variablecanvas;
	/** The table model for varable data. */
	protected VariableTableModel tabelmodel;

	// Top panel
	JTextArea txtareaName;
	JComboBox combo;
	Checkbox chkboxPrescr;
	Checkbox chkboxMeasured;
	String szUnitName = "Unit not set";
	JLabel labelUnit;
	// Import button
	JButton btnImport;
	// Buttom Copy To All
	JButton btnCopyToAll;
	// Indentification
	JTextArea txtareaDataSource;

	// Tabel panel
	JLabel labelErrMsg;
	// Buttons panel
	JButton btnOK;

	/** Creates a dialog for editing variable properties.
	* @param owner The owner of the dialog.
	* @param szTitle The title of the dialog.
	* @param variable The Variable to be edited. */
	public VariableDialog(JFrame owner, String szTitle, Variable variable) {
		super(owner, szTitle, true);
		location = owner.getLocationOnScreen();
		vrunits = ECF.vrunits;
		this.variable = variable;
		init();
	} // VariableDialog

	/** Creates a dialog for editing variable properties.
	* @param owner The owner of the dialog.
	* @param szTitle The title of the dialog.
	* @param variable The Variable to be edited. */
	public VariableDialog(JDialog owner, String szTitle, Variable variable) {
		super(owner, szTitle, true);
		location = owner.getLocationOnScreen();
		vrunits = ECF.vrunits;
		this.variable = variable;
		init();
	} // VariableDialog

	/** Initializes position and content of the dialog. */
	protected void init() {
		location.translate(20,10);
		setLocation(location);
		setSize(500,580);
		makeContentPane();
		show();
	} // init

	protected void makeContentPane() {

		Container pane = getContentPane();
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS ));


		// Add top panel
		pane.add(makeTopPanel());
		// Add table panel
		pane.add(makeTablePanel());
		// Add plot panel
		pane.add(makePlotPanel());
		// Add buttons panel
		pane.add(makeButtonsPanel());

	} // makeContentPane

	protected JPanel makeTopPanel() {
		// JPanel panelTop = new JPanel(new BorderLayout());

		// Grid bag layout panel
		GridBagLayout layoutmanager = new GridBagLayout();
		JPanel panelTop = new JPanel(layoutmanager);

		GridBagConstraints con = new GridBagConstraints();

		// First column of controls
		//=========================

		// Name
		JLabel labelName = new JLabel("Name:");
		con.gridx = 0; con.gridy = 0;
		layoutmanager.setConstraints(labelName,con);
		panelTop.add(labelName);

		// Space label
		JLabel labelSpaceRow1 = new JLabel(" ");
		con.gridx = 0; con.gridy = 1;
		layoutmanager.setConstraints(labelSpaceRow1,con);
		panelTop.add(labelSpaceRow1);

		// Space label
		JLabel labelSpaceRow2 = new JLabel(" ");
		con.gridx = 0; con.gridy = 2;
		layoutmanager.setConstraints(labelSpaceRow2,con);
		panelTop.add(labelSpaceRow2);

		// Units label
		JLabel labelUnits = new JLabel("Units:");
		con.gridx = 0; con.gridy = 3;
		layoutmanager.setConstraints(labelUnits,con);
		panelTop.add(labelUnits);

		// Data source label
		JLabel labelDataSource = new JLabel("Data Source:");
		con.gridx = 0; con.gridy = 4;
		layoutmanager.setConstraints(labelDataSource,con);
		panelTop.add(labelDataSource);

		// Space label
		JLabel labelSpaceRow5 = new JLabel(" ");
		con.gridx = 0; con.gridy = 5;
		layoutmanager.setConstraints(labelSpaceRow5,con);
		panelTop.add(labelSpaceRow5);

		// Space label
		JLabel labelSpaceRow6 = new JLabel(" ");
		con.gridx = 0; con.gridy = 6;
		layoutmanager.setConstraints(labelSpaceRow6,con);
		panelTop.add(labelSpaceRow6);

		// Second column of controls
		//==========================

		// Text area for entering name
		txtareaName = new JTextArea(variable.getName(),3,20);
		JScrollPane scrollpane1 = new JScrollPane(txtareaName);
		con.gridx = 1;     con.gridy = 0;
		con.gridwidth = 1; con.gridheight = 3;
		con.fill = GridBagConstraints.BOTH;
		layoutmanager.setConstraints(scrollpane1,con);
		panelTop.add(scrollpane1);

		// New fresh GridBagConstraints
		con = new GridBagConstraints();
		// Units panel
		JPanel panelUnits = makeUnitsPanel();
		con.gridx = 1; con.gridy = 3;
		con.anchor = GridBagConstraints.WEST;
		con.weightx = 1;
		layoutmanager.setConstraints(panelUnits,con);
		panelTop.add(panelUnits);

		// Text area for entering data source reference
		txtareaDataSource = new JTextArea(variable.getDataSource(),3,20);
		JScrollPane scrollpane2 = new JScrollPane(txtareaDataSource);
		con.gridx = 1;     con.gridy = 4;
		con.gridwidth = 2; con.gridheight = 3;
		con.fill = GridBagConstraints.BOTH;
		layoutmanager.setConstraints(scrollpane2,con);
		panelTop.add(scrollpane2);

		// Third column of controls
		//=====================================
		// New fresh GridBagConstraints
		con = new GridBagConstraints();

		// Prescribed variable checkbox
		chkboxPrescr = new Checkbox(szCheckBoxPrescribedLegend,((Variable)variable).isPrescribed());
		chkboxPrescr.addItemListener(this);
		con.gridx = 2;  con.gridy = 0; con.anchor = GridBagConstraints.WEST;
		layoutmanager.setConstraints(chkboxPrescr,con);
		panelTop.add(chkboxPrescr);

		// Measured variable checkbox
		chkboxMeasured = new Checkbox(szCheckBoxMeasuredLegend,((Variable)variable).isMeasured());
		chkboxMeasured.addItemListener(this);
		con.gridx = 2;  con.gridy = 1;
		layoutmanager.setConstraints(chkboxMeasured,con);
		panelTop.add(chkboxMeasured);

		// Make an import button
		btnImport = new JButton("Import data");
		btnImport.setToolTipText("Import data as text.");
		btnImport.addActionListener(this);
		con.gridx = 2;  con.gridy = 2;
		layoutmanager.setConstraints(btnImport,con);
		panelTop.add(btnImport);

		// Make a copy to all button
		btnCopyToAll = new JButton("Copy value");
		btnCopyToAll.setToolTipText("Copy value at the selected time to all values.");
		btnCopyToAll.addActionListener(this);
		con.gridx = 2; con.gridy = 3;
		layoutmanager.setConstraints(btnCopyToAll,con);
		panelTop.add(btnCopyToAll);

		return panelTop;
	} // makeTopPanel

	protected JPanel makeUnitsPanel() {

		// Units panel
		JPanel panelUnits= new JPanel(new GridLayout(1,3));

		// Units selection combo box
		int ilayerVariable = variable.getLayerIndex();
		combo = new JComboBox();
		Iterator iter = vrunits.iterator();
		int ientry = 0; // Index of entry in this combo box
		while (iter.hasNext()) {
			Units units = (Units)iter.next();
			int ilayerUnits = units.getLayerIndex();
			if (ilayerVariable==ilayerUnits) {
				combo.addItem(units.getName());
				if (units==variable.getUnits()) combo.setSelectedIndex(ientry);
				ientry++;
			} // if
		} // while
		combo.addActionListener(this);
		panelUnits.add(combo,BorderLayout.WEST);

		// Units used
		labelUnit= new JLabel(szUnitName);
		Units units = variable.getUnits();
		if (units!=null) {
			String szUnitName = units.getNameOfDisplayUnit();
			labelUnit.setText(szUnitName);
			combo.setSelectedItem(units.getName());
		} // if
		panelUnits.add(labelUnit,BorderLayout.CENTER);

		return panelUnits;

	} // makeUnitsPanel

	protected JPanel makeTablePanel() {

		// Make table
		tabelmodel = new VariableTableModel(this,(Variable)variable);
		JTable table = new JTable(tabelmodel);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		// Create the scroll pane and add the table to it.
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setToolTipText("If decimal point does not work, try decimal comma.");
		labelErrMsg = new JLabel(" ");
		labelErrMsg.setForeground(Color.red);
		JPanel panelTabel = new JPanel();
		panelTabel.setLayout(new BorderLayout());
		panelTabel.add(scrollPane,BorderLayout.CENTER);
		panelTabel.add(labelErrMsg,BorderLayout.SOUTH);
		panelTabel.setToolTipText("If decimal point does not work, try decimal comma.");
		return panelTabel;

	} // makeTablePanel

	protected JPanel makePlotPanel() {

		// Panel for painting variable plot
		JPanel panelPlot = new JPanel(new BorderLayout());
		// Make canvas for drawing variable plot
		variablecanvas = new VariableCanvas(getSize().width,(Variable)variable);
		panelPlot.add(variablecanvas,BorderLayout.CENTER);
        panelPlot.setBorder(BorderFactory.createTitledBorder(szCanvasLegend));

		return panelPlot;
	} // makePlotPanel

	protected void setVaribleMeasured(boolean fMeasured) {
		variable.setMeasured(fMeasured);
		chkboxMeasured.setState(fMeasured);
	} // setVaribleMeasured

	protected JPanel makeButtonsPanel() {

		JPanel panelButtons = new JPanel();

		// Buttons
		btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		panelButtons.setLayout(new FlowLayout());
		panelButtons.add(btnOK);

		return panelButtons;

	} // makeButtonsPanel

	public void showErrorMsg(String errmsg) {
		labelErrMsg.setText(errmsg);
	} // showErrorMsg

	public void repaint() {
		if (variablecanvas!=null) variablecanvas.repaint();
	} // repaint

	public void actionPerformed(ActionEvent e) {

		// Combo box
		// Set Units of variable according to selection in Combo box
		if (e.getSource()==combo) {
			String szComboName = (String)combo.getSelectedItem();
			Iterator iter = vrunits.iterator();
			while (iter.hasNext()) {
				Units units = (Units)iter.next();
				String szUnitsName = units.getName();
				if (szUnitsName.compareTo(szComboName)==0) {
					labelUnit.setText(units.getNameOfDisplayUnit());
					variable.setUnits(units);
				} // if
			} // while
			repaint();
		} // Units combobox

		// Import button
		if (e.getSource()==btnImport) {
			variable.setName(txtareaName.getText());
			variable.setDataSource(txtareaDataSource.getText());
			new ImportDialog(this,"Import data",(Variable)variable);
			tabelmodel.updateTable();
		} // Import button

		// Import button
		if (e.getSource()==btnCopyToAll) {
			variable.setName(txtareaName.getText());
			variable.setDataSource(txtareaDataSource.getText());
			variable.copyToAll(variable.getTimeAxis().getSelectedIncrement());
			tabelmodel.updateTable();
		} // Import button

		// OK button
		if (e.getSource()==btnOK) {
			variable.setName(txtareaName.getText());
			variable.setDataSource(txtareaDataSource.getText());

			// Prescribed variable or default
			if (chkboxPrescr.getState()) variable.setPrescribed();
			else variable.setDefault();
			// Measured state
			variable.setMeasured(chkboxMeasured.getState());

			dispose();
		} // OK button

	} // actionPerformed

	public void itemStateChanged(ItemEvent e) {
		// Prescribed check box
		if (chkboxPrescr.getState()) variable.setPrescribed();
			else variable.setDefault();
		// Measured check box
		// Set measured state of variable according to Measured in Combo box
		if (e.getSource()==chkboxMeasured) {
			variable.setMeasured(chkboxMeasured.getState());
		} // chkboxMeasured
		tabelmodel.updateTable();
	} // itemStateChanged(ItemEvent e)

} // class VariableDialog