import java.util.*;         // Vector
import javax.swing.table.*; // DefaultTableModel
import javax.swing.event.*; // TableModelEvent
import java.text.ParseException;

/**
* The VariableTableModel stores the values
* of a <A href="Variable.html">Variable</A> to be displayed by a
* VariableDialog.
*/

public class VariableTableModel extends DefaultTableModel  implements TableModelListener {

	protected VariableDialog dialog;
	protected Variable variable;
	protected Vector vrnameColumn = new Vector();
	protected Vector vrData = new Vector();
	protected Vector vrVariableCalculated = new Vector();
	protected Vector vrVariablePrescribed = new Vector();
	protected double vrvalueCalculated[];
	protected double vrvaluePrescribed[];

	/**
	* Creates a VariableTableModel that that is used by a VariableDialog to
	* display an edit the values of a Variable.
	*/

    public VariableTableModel(VariableDialog dialog, Variable variable) {

		this.dialog = dialog;
		this.variable = variable;

		// Set row labels
		vrnameColumn.add("Time");
		// Prescribed row
		String szState = "";
		if (variable.isMeasured()) szState = "Measured";
		if (variable.isPrescribed()) szState = "Prescribed";
		vrVariablePrescribed.add(szState);
		// Calculated row
		vrVariableCalculated.add("Calculated");

		int ccol = variable.size();
		vrvaluePrescribed = new double[ccol];
		Vector vrtimeInc = variable.getTimeAxis().getTimeIncrements();
		double[][] mxvalueDisplay = variable.getDisplayValues();

		// Set column labels and data
		for (int icol=0; icol<ccol; icol++) {
			TimeIncrement timeinc = (TimeIncrement)vrtimeInc.elementAt(icol);
			String szTime = String.valueOf(timeinc.getStartTime());
			vrnameColumn.add(szTime);
			vrVariableCalculated.add(Quantity.formatDouble(mxvalueDisplay[1][icol]));
			vrVariablePrescribed.add(Quantity.formatDouble(mxvalueDisplay[0][icol]));
		} // for icol

		// Set up tabel data
		vrData.add(vrVariablePrescribed);
		vrData.add(vrVariableCalculated);
		setDataVector(vrData,vrnameColumn);
		// Add listener
		addTableModelListener(this);

	} // VariableTableModel

    /*
     * JTable uses this method to determine the default renderer/
     * editor for each cell.
     */
    public Class getColumnClass(int icol) {
		try {
        	return getValueAt(0, icol).getClass();
		} catch(Exception e) {
			System.out.println(e);
			return null;
		} // try
    } // getColumnClass

    /*
     * Don't need to implement this method unless your table's
     * editable.
     */
    public boolean isCellEditable(int irow, int icol) {
        //Note that the data/cell address is constant,
        //no matter where the cell appears onscreen.
//        if (icol==0 || irow==1 || !variable.isPrescribed()) {
        if (icol==0 || irow==1) {
            return false;
        } else {
            return true;
        }
    } // isCellEditable

    /** Update table prescribed values. */
    public void updateTable() {
		// Refresh vector of prescribed values
		vrVariablePrescribed.clear();
		String szState = "";
		if (variable.isMeasured()) szState = "Measured";
		if (variable.isPrescribed()) szState = "Prescribed";
		vrVariablePrescribed.add(szState);
		// Get both prescribed and calculated values
		double[][] mxvalueDisplay = variable.getDisplayValues();
		// Fill vector of prescribed values with new data
		int ccol = variable.size();
		for (int icol=0; icol<ccol; icol++) {
			// Sort out prescribed values only
			vrVariablePrescribed.add(Quantity.formatDouble(mxvalueDisplay[0][icol]));
		} // for icol
		fireTableDataChanged();
	} // updateTable

	/** Update the variable with new data after entering new table data.
	* @param e The event to be forwarded. */
	// Method of interface TableModelListener
    public void tableChanged(TableModelEvent e) {
		int ccol = vrnameColumn.size();
		String szCell = "0.0";
		Vector vrtimeInc = variable.getTimeAxis().getTimeIncrements();
		for (int icol=0; icol<ccol-1; icol++) {
			TimeIncrement timeinc = (TimeIncrement)vrtimeInc.elementAt(icol);
			String szTime = String.valueOf(timeinc.getStartTime());
			try {
				szCell = (String)vrVariablePrescribed.elementAt(icol+1);
				vrvaluePrescribed[icol] = Quantity.parseDouble(szCell);
				dialog.showErrorMsg(" ");
			} catch(ParseException pex) {
				dialog.showErrorMsg("\"" + szCell + "\" at time \"" +
					szTime + "\" is not a valid decimal number.");
				// table.setSelectionForeground(Color.red);
				break;
			} catch(Exception ex) {
				dialog.showErrorMsg("Unexpected error while parsing \"" + szCell +
				"\" at time \"" + szTime + "\": " + ex);
				break;
			} // try/catch
		} // for
		try {
			variable.setDisplayValuesPrescribed(vrvaluePrescribed); // Convert to base values in Variable
			dialog.repaint();
		} catch(Exception ex) {
			System.out.println("VariableTableModel.tableChanged: variable=" + ex);
		}
	} // tableChanged

} // VariableTableModel