import java.util.*;

/**
* The XMLParser decodes one level of a XML code. The structure:
* <p><pre>
*          &lttag&gt
*             sub
*          &lt/tag&gt
*          remainder
* </pre>
* is decompose in its parts <br>
* - tag = the tag of the first item <br>
* - sub = the XML code for the first sublevel of the first part <br>
* - remainder = XML code for the following items, if any
*/
public class XMLParser {

	/** Tag of first XML item. */
	protected String szTagFirst = "first";
	/** Tag of next XML item. */
	protected String szTagNext = "next";
	/** First subcode after first XML tag. */
	protected String szSub = "";
	/** The reminader of XML code. */
	protected String szRemainder = "remainder";
	/** Index of first character of a tag. */
	private int ich1;
	/** Index of last character of a tag. */
	private int ich2;
	/** The index of the code level. */
	private int ilevel = 0;
	/** Index of first character of subcode. */
	private int ichSub1;
	/** Index of last character of subcode. */
	private int ichSub2;

	/**
	* Create a XMLParser for the first level of XML code.
	* @param szXMLCode The complete XML code.
	*/
	public XMLParser(String szXMLCode) throws XMLParserException {
//		System.out.println("XMLParser:\n" + szXMLCode);

		// Find first tag
		ich1 = szXMLCode.indexOf("<");
		ich2 = szXMLCode.indexOf(">");
		int lSz = szXMLCode.length();
		String szXMLCodeStart =
			"at XML-code starting with " + szXMLCode.substring(0,Math.min(lSz,40));
//		System.out.println("XMLParser: ich1=" + ich1 + " ich2=" + ich2);
		if (ich1==-1 || ich2==-1)  {
			String szMsg = "Brackets < or > of first tag of XML code not found.\n" +
				szXMLCodeStart;
			throw new XMLParserException(szMsg);
		} // if
		try {
			szTagFirst = szXMLCode.substring(ich1+1,ich2);
//			System.out.println("XMLParser: szTagFirst=" + szTagFirst);
			ilevel = 1;
			ichSub1 = ich2+1;
		} catch(Exception e) {
			String szMsg = "First tag of XML code not found.\n" +
				szXMLCodeStart;
			throw new XMLParserException(szMsg);
		} // try/catch

		// Parse code levels until level 0 is reached
		try {
			parseOneLevel(szXMLCode);
			while (ilevel>0 && ich2!=-1) {
				parseOneLevel(szXMLCode);
			} // while
		} catch(Exception e) {
			System.out.println(e);
		} // try/catch

		// Find end tag
		String szTagEnd = szTagNext;
//		System.out.println("XMLParser: szTagEnd=" + szTagEnd);
		if (szTagEnd.compareTo("/"+szTagFirst)==0) {
			// End tag matches first tag
			try {
				// Last character of subcode
				ichSub2 = ich1;
				szSub = szXMLCode.substring(ichSub1,ichSub2).trim();
//				System.out.println("XMLParser: szSub = \n" + szSub);
			} catch(Exception e) {
				String szMsg = "\nFirst tag of XML code was " + szTagFirst + "\n" +
					"The corresponding end tag " + szTagEnd + " was not found.";
				throw new XMLParserException(szMsg);
			} // try/catch
		} // if

		// Extrakt remainder
		szRemainder = szXMLCode.substring(ich2+1).trim();
	} // XMLParser

	/** Parse one level. */
	private void parseOneLevel(String szXMLCode) throws XMLParserException {
		try {
			ich1 = szXMLCode.indexOf("<",ich2+1);
			ich2 = szXMLCode.indexOf(">",ich2+1);
			szTagNext = szXMLCode.substring(ich1+1,ich2);
			String szCharFirst = szTagNext.substring(0,1);
//			System.out.println("XMLParser: szTagNext=" + szTagNext +
//				" szCharFirst=" + szCharFirst);
			if (szCharFirst.compareTo("/")==0)
				ilevel--;
			else
				ilevel++;
//			System.out.println("XMLParser: ilevel=" + ilevel);
		} catch(Exception e) {
			System.out.println("XMLParser: ich1=" + ich1 + " ich2=" + ich2 +
				"\nszXMLCode=" + szXMLCode);
			System.out.println("XMLParser: " + e.getMessage());
         	e.printStackTrace();
         	System.exit(3);
//			String szMsg = "Tag of XML code not found.";
//			throw new XMLParserException(szMsg);
		} // try/catch
	} // parseOneLevel

	/**
	* Get the remainder of the XML code.
	* @return The remainder of the XML code. */
	public String remainder() { return szRemainder; }

	/**
	* Get the first sublevel of the first XML item.
	* @return The XML code for the first sublevel. */
	public String sub() {
//		System.out.println("XMLParser: szSub=\n  " + szSub + "\n");
		return szSub;
	} // sub

	/**
	* Get the tag of the first XML item.
	* @return The tag of the first XML item. */
	public String tag() { return szTagFirst; }

	/** Test of XML parser.
	* @param args Not used. */
	public static void main(String[] args) {
		System.out.println("XMLParser.main:");

		/** The XML parser. */
		XMLParser parser = null;
		/** XML test code. */
		String szXMLCode = "<XML>\n" +
		                   "  <XML>\n" +
		                   "  </XML>\n" +
		                   "  <sector>\n" +
		                   "  </sector>\n" +
		                   "</XML>\n" +
		                   "<Remainder>\n" +
		                   "  rem1\n" +
		                   "</Remainder>\n" +
		                   "<Remainder>\n" +
		                   "  rem2\n" +
		                   "</Remainder>\n";
		// Parse XML code
		while (szXMLCode.length()>0) {
			try {
				parser = new XMLParser(szXMLCode);
			} catch (XMLParserException e) {
				System.out.println("XMLParser.main:" + e);
				break;
			} // try/catch
			// Find remainder and use it for decoding next level
			String szTag = parser.tag();
			String szSub = parser.sub();
			szXMLCode = parser.remainder();
			System.out.println("XMLParser.main: szTag=" + szTag + "\n");
			System.out.println("XMLParser.main: szSub=\n  " + szSub + "\n");
			System.out.println("XMLParser.main: remainder=\n" + szXMLCode + "\n");
		} // while

	} // main

} // XMLParser